<module/bit/
<section/Synopsis/

[[
cheia.load "bit"

r = bit.bnot(x)
r = bit.band(a, b, ...)
r = bit.bor(a, b, ...)
r = bit.bxor(a, b, ...)
r = bit.lshift(x, places)
r = bit.rshift(x, places)
r = bit.arshift(x, places)
r = bit.mod(numerator, denominator)
]]

<section/Description/

This module provides a selection of bitwise operations.

<warn>Beware of precision. Lua stores all numbers in double-precision
floating-point format. This means that large numbers are not stored
accurately. Therefore, the functions in this module should not be
expected to behave correctly for numbers exceeding 32 bits (although
53 bits are probably safe at least on i386). This is of particular
consequence for the [bit.lshift] operation: the most significant bits
are not discarded, but shifted with the rest, which may cause the
number to grow large enough that precision is lost for the least
significant bits. For this reason, a [bit.lshift] should always be
preceded by a [bit.mod] to discard the unwanted bits when there is any
danger of the number growing too large.</warn>

<note>The fractional part of any operand is discarded. The right shift
operations do not move bits into the fractional part.</note>

<section/Examples/

To clear bit [n] (0-based) of number [x]:

[[
x = bit.band(bit.bnot(bit.lshift(1, n)), x)
]]

To retrieve the high and low bytes of unsigned 16-bit number [x]:

[[
high = bit.rshift(x, 8)
low = bit.mod(x, 256)
]]

<section/Reference/

All operations are directly implemented using their C equivalents operating on g[long long]s.

doc[[

=fn bit.bnot
=in x number	Operand.
=out number	Result.
=d Perforns a bitwise NOT of its operand. Equivalent to g[~] in C.

=fn bit.band
=in ... numbers Two or more operands.
=out number	Result.
=d Performs a bitwise AND of its operands. Equivalent to g[&] in C.

=fn bit.bor
=in ... numbers Two or more operands.
=out number	Result.
=d Performs a bitwise OR of its operands. Equivalent to g[|] in C.

=fn bit.bxor
=in ... numbers Two or more operands.
=out number	Result.
=d Performs a bitwise XOR of its operands. Equivalent to g[^] in C.

=fn bit.lshift
=in x number	Operand.
=in n number	Number of places to shift.
=d Performs a bitwise left shift of its operand. Equivalent to g[<<] in C. See the warning above.

=fn bit.rshift
=in x number	Operand.
=in n number	Number of places to shift.
=d Performs an unsigned bitwise right shift of its operand. The result
is always positive. Equivalent to g[>>] in C.

=fn bit.arshift
=in x number	Operand.
=in n number	Number of places to shift.
=d Performs a bitwise right shift of its operand, but the sign of the
operand is preserved. Equivalent to g[>>] in C.

=fn bit.mod
=in n number	Numerator.
=in d number	Denominator.
=d Returns the remainder of an integer division of [n] by [d]. Equivalent to g[%] in C.

]]

<section/Issues/

Should the module automatically clip numbers to within the <q/safe/ range?

Since all <q/safe/ numbers fit within a g[signed long long], there
seems no need to distinguish [bit.rshift] and [bit.arshift].

<section/See also/

None.

<section/Revision history/

Added in LuaCheia 5.0.

<section/Credits/

Lua module by Reuben Thomas.

Modifications for LuaCheia by Thatcher Ulrich.

Documentation for LuaCheia by Jamie Webb.
