<!--*- coding: utf-8 -*-->
<module/cgi/
<section/Synopsis/

[[
cheia.load "cgi"

data = cgi.process()

cgi.put_header()
]]

<section/Description/

This module provides functions helpful for writing CGI (Common Gateway
Interface) web applications. CGI is a de facto standard supported by
almost all web servers which allows web pages to be generated on
request by a program. Communication between the web server and the CGI
program is via environment variables and the standard input and output
streams.

Typically, a web browser provides parameters to a CGI program using a query URL of the form
path[http://www.example.com/cgi-bin/example.lua?key1=value1&key2=value2].
The web server in this case will execute path[example.lua], using the
LuaCheia interpreter if it is set up to do so (e.g. using a <q/tt[#!]/
line under UNIX), and pass the portion of the URL following the
<q/?/ to it in the tt[QUERY_STRING] environment variable. The
program acts on this information and sends back an HTML page,
preceded by an HTTP header, to [stdout].

The [cgi] module provides the [cgi.process] function to decode the
query string (or data sent using the alternative POST method) into a
Lua table, and the [cgi.put_header] function to output a standard HTTP
header, or a customised one based on a table. It also provides helper
functions for encoding and decoding data in URL format (i.e. containing
<q/tt[%xx]/ sequences to represent special characters using their
ASCII hexadecimal form), etc.

<section/Examples/

Typically, the following program could be placed in the path[cgi-bin]
directory of a UNIX web server. It simply provides a form with a text box,
and repeats back anything entered.

[[
#!/usr/local/bin/luacheia5

-- Load the cgi module
cheia.load "cgi"

-- Read the submitted data into the 'data' table
data = cgi.process()

\--- Start writing the output page
\+cgi.put_header()
print("<html><body>")

-- If the form was submitted, print the result
if data.out then
        print("<p>You submitted: " .. data.out .. "</p>")
end

-- Print the form
print [[
<form method="GET" action="test.cgi">
Enter something: <input name="out" type="text"></input>
<input type="submit"></input>
</form>
</body>
</html>
]]
]]

<section/Reference/

doc[[

=fn cgi.put_header
=inopt header table	Specifies the header to use. If absent, the default header is used.
=inopt fd file		The file descriptor to write to. Defaults to [io.stdout].
=d Writes out an HTTP header. This function must be called before any
text is sent. If the [header] parameter is specified, it should be a table of header entries. Currently, the following keys are supported:

defs[[
<b>Table key</b> | <b>HTTP header entry</b>

[content] | Content-type

[redirect] | Location
]]

The default header is simply <q>Content-type: text/html</q>.

=fn cgi.url_decode
=in text string		The text to decode.
=inopt use_plus boolean	If true or absent, allows a <q/tt[+]/ to represent
a space in the input.
=out string		The decoded string.
=d Decodes a URL-encoded string.

=fn cgi.url_encode
=in text string		The text to encode.
=inopt use_plus boolean	If true or absent, uses <q/tt[+]/ rather than
<q/tt[%20]/ to represent a space.
=out string		The encoded string.
=d URL-encodes a string.

=fn cgi.encode_qs
=in tbl table		The key/value pairs to encode.
=out string		The equivalent query string.
=d Encodes a table for use in a query URL. The returned string does not
include the leading <q/tt[?]/ character.

=fn cgi.process
=inopt fd file		The file descriptor to read POST requests from. Defaults to
[io.stdin].
=out table		The decoded key/value pairs.
=d Detects the request type and processes the data sent in the request
appropriately. Returns a table of the parameters. Also sets the
following variables based on the environment:

defs[[
[cgi.QUERY_STRING] | The part of the URL following the <q/?/ in a
GET request, if any.

[cgi.PATH_INFO] | The part of the URL path following the script name,
if any (used if the CGI program is treated as a <q/virtual directory/).

[cgi.REQUEST_METHOD] | The type of request, e.g. GET, HEAD, POST...

[cgi.CONTENT_TYPE] | The encoding used to send data in the body of
e.g. a POST request. Usually either
path[application/x-www-form-urlencoded] or path[multipart/form-data].
]]

]]

<subsection/Additional Functions/

These functions are primarily for internal use.

doc[[

=fn cgi.strip_rr
=in text string		Input.
=out string		Result.
=d Strips CR characters from a string.

=fn cgi.strip_nn
=in text string		Input.
=out string		Result.
=d Strips LF characters from a string.

=fn cgi.strip_rn
=in text string		Input.
=out string		Result.
=d Strips CR and LF characters from a string.

=fn cgi.process_multipart_post
=inopt fd file		The file descriptor to read from. Defaults to
[io.stdin].
=out table		The decoded key/value pairs.
=d Reads the body of an HTTP POST request which has been encoded using
g[multipart/form-data].

=fn cgi.decode_qs
=in qs string		The query string.
=inopt tbl table	The output table. If absent, a new one is
created.
=out table		A table containing the decoded key/value
pairs.
=d Decodes a query string into a table.

=fn cgi.process_urlenc_post
=inopt fd file		The file descriptor to read from. Defaults to
[io.stdin].
=out table		The decoded key/value pairs.
=d Reads the body of an HTTP POST request which has been encoded using
path[application/x-www-form-urlencoded] (the default).

=fn cgi.process_get
=out table		The decoded key/value pairs.
=d Reads and decodes the tt[QUERY_STRING] envirnment variable, as
supplied by an HTTP GET request.

]]

<section/Issues/

Does not handle multiple values for the same key.

Is this thing portable? Seems to assume we want UNIX line-endings.

URL encoding looks broken to me (plus sign handling).

[__header_items] seems like unnecessary obfuscation.

<section/See also/

The W3C CGI page, url[http://www.w3.org/cgi/]. (CGI is not a
W3C standard.)

<section/Revision history/

Added in LuaCheia 5.0.

<section/Credits/

Written by Adrián Pérez de Castro.

Documentation for LuaCheia by Jamie Webb.
