<?xml version="1.0" encoding="ISO-8859-1" ?><!--*- sgml; tab-width: 2 -*-->
<xsl:stylesheet version="1.0"
		xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

	<xsl:output method="html"/>

<!--*****************************************************-->
<!--	Setup																							 -->
<!--*****************************************************-->

	<xsl:template match="/xfile">
		<xsl:call-template name="newfile">
			<xsl:with-param name="filename">index</xsl:with-param>
			<xsl:with-param name="title"><xsl:apply-templates select="document" mode="read" /></xsl:with-param>
			<xsl:with-param name="type">top</xsl:with-param>
			<xsl:with-param name="toc">top</xsl:with-param>
		</xsl:call-template>
	</xsl:template>

	<xsl:template name="newfile">
		<xsl:param name="filename" />
		<xsl:param name="type" />
		<xsl:param name="title" />
		<xsl:document href="{$filename}.html">
			<html>
				<head>
					<title><xsl:value-of select="$title"/></title>
					<link rel="stylesheet" type="text/css" href="style.css" />
				</head>
				<body>
					<xsl:choose>
						<xsl:when test="$type = 'top'">
							<h1><xsl:apply-templates select="document" mode="read" /></h1><hr/>
						</xsl:when>
						<xsl:otherwise>
							<h3><xsl:apply-templates select="preceding::document" mode="read" /></h3>
						</xsl:otherwise>
					</xsl:choose>
					<xsl:apply-templates />
				</body>
			</html>
		</xsl:document>
		<xsl:if test="$toc != ''">
			<xsl:if test="not(preceding-sibling::node()[1][self::xfile[chapter|module|appendix]])"><xsl:text disable-output-escaping="yes">&lt;ul class="toc"></xsl:text></xsl:if>
			<li><a href="{$filename}.html"><xsl:value-of select="$toc"/></a></li>
			<xsl:if test="not(following-sibling::node()[1][self::xfile[chapter|module|appendix]])"><xsl:text disable-output-escaping="yes">&lt;/ul></xsl:text></xsl:if>
		</xsl:if>
	</xsl:template>

<!--*****************************************************-->
<!--	Structure																					 -->
<!--*****************************************************-->

	<xsl:template match="xfile[module]">
		<xsl:call-template name="newfile">
			<xsl:with-param name="filename"><xsl:choose><xsl:when test="module/@id"><xsl:value-of select="module/@id" /></xsl:when><xsl:otherwise><xsl:value-of select="module/text()" /></xsl:otherwise></xsl:choose></xsl:with-param>
			<xsl:with-param name="title"><xsl:apply-templates select="module/node()" /> - <xsl:apply-templates select="preceding::document" mode="read" /></xsl:with-param>
			<xsl:with-param name="toc"><xsl:apply-templates select="module/node()" /></xsl:with-param>
		</xsl:call-template>
	</xsl:template>

	<xsl:template match="xfile[appendix]">
		<xsl:call-template name="newfile">
			<xsl:with-param name="filename"><xsl:choose><xsl:when test="appendix/@id"><xsl:value-of select="appendix/@id" /></xsl:when><xsl:otherwise><xsl:value-of select="appendix/text()" /></xsl:otherwise></xsl:choose></xsl:with-param>
			<xsl:with-param name="title"><xsl:apply-templates select="appendix/node()" /> - <xsl:apply-templates select="preceding::document" mode="read" /></xsl:with-param>
			<xsl:with-param name="toc"><xsl:apply-templates select="appendix/node()" /></xsl:with-param>
		</xsl:call-template>
	</xsl:template>

	<xsl:template match="xfile[chapter]">
		<xsl:call-template name="newfile">
			<xsl:with-param name="filename"><xsl:choose><xsl:when test="chapter/@id"><xsl:value-of select="chapter/@id" /></xsl:when><xsl:otherwise><xsl:value-of select="chapter/text()" /></xsl:otherwise></xsl:choose></xsl:with-param>
			<xsl:with-param name="title"><xsl:apply-templates select="chapter/node()" /> - <xsl:apply-templates select="preceding::document" mode="read" /></xsl:with-param>
			<xsl:with-param name="toc"><xsl:apply-templates select="chapter/node()" /></xsl:with-param>
		</xsl:call-template>
	</xsl:template>

	<xsl:template match="xfile"><xsl:apply-templates /></xsl:template>
	<xsl:template match="document" mode="read"><xsl:apply-templates /></xsl:template>
	<xsl:template match="document"></xsl:template>

	<xsl:template match="part"><h2><xsl:apply-templates /></h2></xsl:template>

	<xsl:template match="module">
		<h1><xsl:apply-templates /></h1>
	</xsl:template>

	<xsl:template match="appendix">
		<h1><xsl:apply-templates /></h1>
	</xsl:template>

	<xsl:template match="chapter">
		<h1><xsl:apply-templates /></h1>
	</xsl:template>

	<xsl:template match="section">
		<h2><xsl:apply-templates /></h2>
	</xsl:template>

	<xsl:template match="subsection">
		<h3><xsl:apply-templates /></h3>
	</xsl:template>

	<xsl:template match="ref">
		<xsl:variable name="ref" select="text()"/>
		<xsl:choose>
			<xsl:when test="//appendix[text()=$ref or @id=$ref]">
				<a href="{$ref}.html"><xsl:apply-templates select="//appendix[text()=$ref or @id=$ref]/node()"/></a>
			</xsl:when>
			<xsl:when test="//module[text()=$ref or @id=$ref]">
				<a href="{$ref}.html">Module <i><xsl:value-of select="$ref"/></i></a>
			</xsl:when>
			<xsl:otherwise>[Unknown reference: <i><xsl:value-of select="$ref"/></i>]</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

<!--*****************************************************-->
<!--	Formatting																				 -->
<!--*****************************************************-->

	<xsl:template match="url">
		<xsl:element name="a">
			<xsl:attribute name="href"><xsl:apply-templates /></xsl:attribute>
			<xsl:apply-templates />
		</xsl:element>
	</xsl:template>

	<xsl:template match="manpage">
		<xsl:element name="a">
			<xsl:attribute name="href">http://www.rt.com/man/<xsl:value-of select="@name"/>.<xsl:value-of select="@section"/>.html</xsl:attribute>
			<xsl:value-of select="@name"/>(<xsl:value-of select="@section"/>)</xsl:element>
	</xsl:template>

	<xsl:template match="q"><xsl:text disable-output-escaping="yes">&amp;</xsl:text>lsquo;<xsl:apply-templates /><xsl:text disable-output-escaping="yes">&amp;</xsl:text>rsquo;</xsl:template>
	<xsl:template match="tt"><tt><xsl:apply-templates /></tt></xsl:template>
	<xsl:template match="b"><b><xsl:apply-templates /></b></xsl:template>
	<xsl:template match="path">"<i><xsl:apply-templates /></i>"</xsl:template>
	<xsl:template match="e"><em><xsl:apply-templates /></em></xsl:template>
	<xsl:template match="warn"><b>Warning: </b><xsl:apply-templates /></xsl:template>
	<xsl:template match="fixme"><b>FIXME: </b><xsl:apply-templates /></xsl:template>
	<xsl:template match="note"><b>Note: </b><xsl:apply-templates /></xsl:template>
	<xsl:template match="disp"><div class="disp"><xsl:apply-templates /></div></xsl:template>
	<xsl:template match="ob"></xsl:template>

	<xsl:template match="ul">
		<ul class="std">
			<xsl:apply-templates />
		</ul>
	</xsl:template>

	<xsl:template match="ol">
		<ol class="std">
			<xsl:apply-templates />
		</ol>
	</xsl:template>

	<xsl:template match="ul/para|ul/li|ol/para|ol/li"><li><xsl:apply-templates /></li></xsl:template>

	<xsl:template match="para"><p class="std"><xsl:apply-templates /></p></xsl:template>

<!--*****************************************************-->
<!--	Tables																						 -->
<!--*****************************************************-->

	<xsl:template match="defs">
		<center>
			<table class="defs" width="93%" bgcolor="#f0f0ff" cellpadding="10">
				<tr><td><table cellspacing="10" valign="baseline">
					<xsl:for-each select="para">
						<tr><td><xsl:apply-templates select="def" /></td>
						<td><xsl:apply-templates select="node()[not(self::def)]" /></td></tr>
					</xsl:for-each>
				</table></td></tr>
			</table>
		</center>
	</xsl:template>

<!--*****************************************************-->
<!--	Code																							 -->
<!--*****************************************************-->

	<xsl:template match="code"><i><span class="code"><xsl:apply-templates /></span></i></xsl:template>

	<xsl:template match="codeblock[@style='lua']">
		<center>
			<table class="luacode" width="93%" bgcolor="#ffffe0" cellpadding="10">
				<tr><td><table class="code"><xsl:apply-templates /></table></td></tr>
			</table>
		</center>
	</xsl:template>

	<xsl:template match="codeblock[@style='shell']">
		<center>
			<table class="shell" width="93%" bgcolor="#f7fff0" cellpadding="10">
				<tr><td><table class="code"><xsl:apply-templates /></table></td></tr>
			</table>
		</center>
	</xsl:template>

	<xsl:template match="codeblock[@style='c']">
		<div class="ccode"><table class="ccode"><xsl:apply-templates /></table></div>
	</xsl:template>

	<xsl:template name="insert_space">
		<xsl:param name="num">1</xsl:param>
		<xsl:if test="$num > 0">
			<xsl:text disable-output-escaping="yes">&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;</xsl:text>
			<xsl:call-template name="insert_space">
				<xsl:with-param name="num" select="$num - 1" />
			</xsl:call-template>
		</xsl:if>
	</xsl:template>

	<xsl:template match="codeline[@depth]">
		<tr><td style="white-space: nowrap;" valign="baseline">
			<nobr><xsl:call-template name="insert_space"><xsl:with-param name="num" select="@depth" /></xsl:call-template>
			<xsl:apply-templates select="node()[not(self::linecomment) and not(self::lineresult)]" /></nobr>
		</td><td style="padding-left: 20px;" valign="baseline">
			<xsl:apply-templates select="linecomment|lineresult" />
		</td></tr>
	</xsl:template>

	<xsl:template match="codeline[@empty]">
		<tr><td><xsl:text disable-output-escaping="yes">&amp;nbsp;</xsl:text></td></tr>
	</xsl:template>

	<xsl:template match="codeline[@ignore]"></xsl:template>

	<xsl:template match="keyword">
		<b><xsl:apply-templates /></b>
	</xsl:template>

	<xsl:template match="linecomment"><font color="#9e8a4b"><xsl:apply-templates /></font></xsl:template>
	<xsl:template match="lineresult">
		<font color="#9e8a4b">
			<xsl:text disable-output-escaping="yes">&amp;raquo;</xsl:text>
			<xsl:apply-templates />
		</font>
	</xsl:template>

	<xsl:template match="comment"><font color="#9e8a4b"><xsl:apply-templates /></font></xsl:template>
	<xsl:template match="result">
		<font color="#9e8a4b">
			<xsl:text disable-output-escaping="yes">&amp;raquo;</xsl:text>
			<xsl:apply-templates />
		</font>
	</xsl:template>

	<xsl:template match="string"><font color="#906060"><xsl:apply-templates /></font></xsl:template>

	<xsl:template match="missingcode">...</xsl:template>

<!--*****************************************************-->
<!--	Docs																							 -->
<!--*****************************************************-->

	<xsl:template match="doc"><dl class="code">
		<xsl:apply-templates><xsl:sort select="@name" /></xsl:apply-templates>
	</dl></xsl:template>

	<xsl:template match="function">
		<dt class="doc"><b><i><xsl:apply-templates select="name" />(<xsl:for-each select="in[1]"><xsl:if test="@opt">[</xsl:if><xsl:apply-templates select="name" /><xsl:if test="@opt">]</xsl:if></xsl:for-each><xsl:for-each select="in[position() > 1]">, <xsl:if test="@opt">[</xsl:if><xsl:apply-templates select="name" /><xsl:if test="@opt">]</xsl:if></xsl:for-each>)</i></b></dt>
		<dd class="doc"><xsl:apply-templates select="desc" />
		<xsl:if test="in|out"><table class="text">
			<xsl:if test="in"><tr><td colspan="2">Parameters:</td></tr>
			   <xsl:for-each select="in">
					<tr><td style="white-space: nowrap;" valign="baseline">
						<xsl:call-template name="insert_space" />
						<i><span class="code">
							<xsl:apply-templates select="name" /> : <xsl:apply-templates select="type" />
							<xsl:text disable-output-escaping="yes">&amp;nbsp;&amp;nbsp;</xsl:text>
						</span></i>
					</td><td valign="baseline">
						<xsl:apply-templates select="desc" />
					</td></tr>
				</xsl:for-each>
			</xsl:if>
			<xsl:if test="out"><tr><td colspan="2">Returns:</td></tr>
				<xsl:for-each select="out">
					<tr><td style="white-space: nowrap;" valign="baseline">
						<xsl:call-template name="insert_space" />
						<i><span class="code">
							<xsl:apply-templates select="type" />
							<xsl:text disable-output-escaping="yes">&amp;nbsp;&amp;nbsp;</xsl:text>
						</span></i>
					</td><td valign="baseline">
						<xsl:apply-templates select="desc" />
					</td></tr>
				</xsl:for-each>
			</xsl:if>
		</table></xsl:if>
		</dd>
	</xsl:template>

<!--*****************************************************-->
<!--	Characters																				 -->
<!--*****************************************************-->

	<xsl:template match="char[@name='lbr']">{</xsl:template>
	<xsl:template match="char[@name='rbr']">}</xsl:template>
	<xsl:template match="char[@name='pct']">%</xsl:template>
	<xsl:template match="char[@name='dollar']">$</xsl:template>
	<xsl:template match="char[@name='uscore']">_</xsl:template>
	<xsl:template match="char[@name='doubledash']">--</xsl:template>
	<xsl:template match="char[@name='bslash']">\</xsl:template>
	<xsl:template match="char[@name='tilde']">~</xsl:template>
	<xsl:template match="char[@name='bar']">|</xsl:template>
	<xsl:template match="char[@name='hat']">^</xsl:template>
	<xsl:template match="char"><xsl:text disable-output-escaping="yes">&amp;</xsl:text><xsl:value-of select="@name" />;</xsl:template>
	<xsl:template match="char[@name='hash']">#</xsl:template>
</xsl:stylesheet>
