<?xml version="1.0" encoding="UTF-8" ?><!--*- sgml -*-->
<xsl:stylesheet version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

    <xsl:output method="text"/>

<!--*****************************************************-->
<!--	Setup 						 -->
<!--*****************************************************-->

    <xsl:template match="/xfile">\documentclass{book}
\usepackage{charter}
\usepackage[a4paper]{geometry}
\usepackage{ifthen}
\usepackage{makeidx}
\usepackage[utf8/utf8]{inputenc}

\setlength\parskip{\medskipamount}
\setlength\parindent{0pt}

\ifx\pdfoutput\undefined
  \usepackage[dvips]{graphicx}
  \DeclareGraphicsExtensions{.eps,.ps,.bmp,.tif,.tiff,.tga}  
\else
  \pdfoutput=1
  \pdfcompresslevel=9
  \usepackage[pdftex]{graphicx}
  \DeclareGraphicsExtensions{.jpg,.jpeg,.pdf,.mps,.png} 
  \usepackage{color}
  \definecolor{LinkBlue}{rgb}{0,0.08,0.45}
  \definecolor{LinkRed}{rgb}{0.45,0.08,0}
  \usepackage[colorlinks,urlcolor=LinkBlue,linkcolor=LinkRed]{hyperref}
\fi

\newlength{\codewidth}
\newlength{\testwidth}
\newcommand{\resetwidth}{\setlength{\codewidth}{0pt}}
\newcommand{\applywidth}[1]{
    \settowidth{\testwidth}{#1}
    \ifthenelse{\lengthtest{\testwidth>\codewidth}}{\setlength{\codewidth}{\testwidth}}{}}

\newenvironment{varlist}
  {\begin{list}{}
    {\setlength{\labelwidth}{\codewidth}
     \setlength{\leftmargin}{\labelwidth}
     \addtolength{\leftmargin}{\labelsep}
     \renewcommand{\makelabel}[1]{##1\hfil}}}
  {\end{list}}

\newenvironment{codeenv}
  {\begin{list}{}
    {\setlength{\labelwidth}{\codewidth}
     \setlength{\leftmargin}{\labelwidth}
     \addtolength{\leftmargin}{\labelsep}
     \setlength{\itemsep}{-3pt}
     \renewcommand{\makelabel}[1]{##1\hfil}}}
  {\end{list}}

\newenvironment{fixedlist}[1]
  {\begin{list}{}
    {\settowidth{\labelwidth}{#1}
     \setlength{\leftmargin}{\labelwidth}
     \addtolength{\leftmargin}{\labelsep}
     \renewcommand{\makelabel}[1]{##1\hfil}}}
  {\end{list}}

\ifx\pdfoutput\undefined
  \newcommand{\extlink}[2]{[\textsl{#2}]}
\else
  \newcommand{\extlink}[2]{[\href{#1}{\textrm{\textsl{#2}}}]}
\fi
\newcommand{\ttilde}{\symbol{126}}

\newcommand{\sectionnotoc}[1]{%
  \refstepcounter{section}%
  \section*{\thesection\hspace{2ex}#1}%
}

\newcommand{\subsectionnotoc}[1]{%
  \refstepcounter{subsection}%
  \subsection*{\thesubsection\hspace{2ex}#1}%
}

\makeindex
\title{<xsl:apply-templates select="document" mode="read" />}
\author{}
\begin{document}
\maketitle
\tableofcontents

<xsl:apply-templates />

\backmatter
\addcontentsline{toc}{chapter}{\indexname}
\printindex

\end{document}</xsl:template>

<!--*****************************************************-->
<!--	Structure					 -->
<!--*****************************************************-->

    <xsl:template match="xfile"><xsl:apply-templates /></xsl:template>
    <xsl:template match="document" mode="read"><xsl:apply-templates /></xsl:template>
    <xsl:template match="document"></xsl:template>

    <xsl:template match="part">\part{<xsl:apply-templates />}

</xsl:template>

    <xsl:template match="chapter">\chapter{<xsl:apply-templates />}\label{<xsl:choose><xsl:when test="@id"><xsl:value-of select="@id"/></xsl:when><xsl:otherwise><xsl:value-of select="text()"/></xsl:otherwise></xsl:choose>}

</xsl:template>

    <xsl:template match="module">\chapter{<xsl:apply-templates />}\index{<xsl:apply-templates />}\label{<xsl:choose><xsl:when test="@id"><xsl:value-of select="@id"/></xsl:when><xsl:otherwise><xsl:value-of select="text()"/></xsl:otherwise></xsl:choose>}

</xsl:template>

    <xsl:template match="appendix"><xsl:if test="not(preceding::appendix)">\appendix</xsl:if>\chapter{<xsl:apply-templates />}\index{<xsl:apply-templates />}\label{<xsl:choose><xsl:when test="@id"><xsl:value-of select="@id"/></xsl:when><xsl:otherwise><xsl:value-of select="text()"/></xsl:otherwise></xsl:choose>}

</xsl:template>

    <xsl:template match="section">\section<xsl:if test="preceding::node()[@contents-level][1]/@contents-level &lt; 1">notoc</xsl:if>{<xsl:apply-templates />}

</xsl:template>

    <xsl:template match="subsection">\subsection<xsl:if test="preceding::node()[@contents-level][1]/@contents-level &lt; 2">notoc</xsl:if>{<xsl:apply-templates />}

</xsl:template>

    <xsl:template match="ref"><xsl:variable name="ref" select="text()"/><xsl:choose><xsl:when test="//appendix[text()=$ref or @id=$ref]">Appendix \ref{<xsl:value-of select="$ref"/>} (page \pageref{<xsl:value-of select="$ref"/>})</xsl:when><xsl:when test="//module[text()=$ref or @id=$ref]">Module \textsl{<xsl:value-of select="$ref"/>} (page \pageref{<xsl:value-of select="$ref"/>})</xsl:when><xsl:otherwise>[Unknown reference: \textsl{<xsl:value-of select="$ref"/>}]</xsl:otherwise></xsl:choose></xsl:template>

<!--*****************************************************-->
<!--	Formatting					 -->
<!--*****************************************************-->

    <xsl:template match="url">\extlink{<xsl:apply-templates mode="read"/>}{<xsl:apply-templates />}</xsl:template>
    <xsl:template match="manpage">\textsl{<xsl:value-of select="@name"/>(<xsl:value-of select="@section"/>)}</xsl:template>
    <xsl:template match="path">``\textsl{<xsl:apply-templates />}''</xsl:template>
    <xsl:template match="q">`<xsl:apply-templates />'</xsl:template>
    <xsl:template match="tt">\texttt{<xsl:apply-templates />}</xsl:template>
    <xsl:template match="b">\textbf{<xsl:apply-templates />}</xsl:template>
    <xsl:template match="e">\emph{<xsl:apply-templates />}</xsl:template>
    <xsl:template match="warn">\textbf{Warning:} <xsl:apply-templates /></xsl:template>
    <xsl:template match="fixme">\textbf{FIXME:} <xsl:apply-templates /></xsl:template>
    <xsl:template match="note">\textbf{Note:} <xsl:apply-templates /></xsl:template>
    <xsl:template match="ob">\linebreak[0]</xsl:template>

    <xsl:template match="disp">\begin{quote}
<xsl:apply-templates />
\end{quote}
</xsl:template>

    <xsl:template match="ul">\begin{itemize}
<xsl:apply-templates />
\end{itemize}
</xsl:template>

    <xsl:template match="ul/para">\item <xsl:apply-templates />
</xsl:template>

    <xsl:template match="ul/li">\item <xsl:apply-templates />
</xsl:template>

    <xsl:template match="para">

<xsl:apply-templates /></xsl:template>

<!--*****************************************************-->
<!--	Tables						 -->
<!--*****************************************************-->

    <xsl:template match="defs">\resetwidth
<xsl:for-each select="para">\applywidth{<xsl:apply-templates select="def" />}
</xsl:for-each>\begin{quote}\begin{varlist}
<xsl:for-each select="para">\item[<xsl:apply-templates select="def" />] <xsl:apply-templates select="node()[not(self::def)]" />
</xsl:for-each>\end{varlist}\end{quote}
</xsl:template>

<!--*****************************************************-->
<!--	Code						 -->
<!--*****************************************************-->

    <xsl:template match="code[@style='lua']">\textsl{<xsl:apply-templates />}</xsl:template>

    <xsl:template match="code[@style='g']">\textsl{<xsl:apply-templates />}</xsl:template>

    <xsl:template match="codeblock[@style='lua']">\resetwidth
<xsl:apply-templates mode="measure" />
\begin{verse}\fbox{\hspace{10pt}\begin{minipage}{0.83\textwidth}\begin{codeenv}\vspace{10pt}<xsl:apply-templates mode="out" />\vspace{10pt}\end{codeenv}\end{minipage}\hspace{10pt}}\end{verse}

</xsl:template>

    <xsl:template match="codeblock[@style='shell']">\resetwidth
<xsl:apply-templates mode="measure" />
\begin{verse}\fbox{\hspace{10pt}\begin{minipage}{0.83\textwidth}\begin{codeenv}\vspace{10pt}<xsl:apply-templates mode="out" />\vspace{10pt}\end{codeenv}\end{minipage}\hspace{10pt}}\end{verse}

</xsl:template>

    <xsl:template match="codeblock[@style='c']">\resetwidth
<xsl:apply-templates mode="measure" />
\begin{verse}\fbox{\hspace{10pt}\begin{codeenv}\vspace{10pt}<xsl:apply-templates mode="out" />\vspace{10pt}\end{codeenv}\hspace{10pt}}\end{verse}

</xsl:template>

    <xsl:template match="codeline[@depth]" mode="out">\item[\textsl{\hspace{<xsl:value-of select="@depth" />cm}<xsl:apply-templates select="node()[not(self::linecomment) and not(self::lineresult)]"/>}] <xsl:apply-templates select="linecomment|lineresult" />~</xsl:template>

    <xsl:template match="codeline[@depth]" mode="measure">\applywidth{\textsl{\hspace{<xsl:value-of select="@depth" />cm}<xsl:apply-templates select="node()[not(self::linecomment) and not(self::lineresult)]"/>}}</xsl:template>

    <xsl:template match="codeline[@empty]" mode="out">\item[~]
</xsl:template>

    <xsl:template match="codeline[@empty]" mode="measure"></xsl:template>
    <xsl:template match="codeline[@ignore]" mode="measure"></xsl:template>
    <xsl:template match="codeline[@ignore]" mode="out"></xsl:template>

    <xsl:template match="keyword">\textbf{<xsl:apply-templates />}</xsl:template>

    <xsl:template match="linecomment">\textit{<xsl:apply-templates />}</xsl:template>
    <xsl:template match="lineresult">$\Rightarrow$ \textsl{<xsl:apply-templates />}</xsl:template>

    <xsl:template match="comment">\textit{<xsl:apply-templates />}</xsl:template>
    <xsl:template match="result">$\Rightarrow$ \textsl{<xsl:apply-templates />}</xsl:template>
    <xsl:template match="string">\textup{<xsl:apply-templates />}</xsl:template>

    <xsl:template match="missingcode">\smash{\raisebox{-1pt}{$\vdots$}}</xsl:template>

<!--*****************************************************-->
<!--	Docs						 -->
<!--*****************************************************-->

    <xsl:template match="doc">\begin{description}
<xsl:apply-templates><xsl:sort select="@name" /></xsl:apply-templates>
\end{description}
</xsl:template>

    <xsl:template match="function">\index{<xsl:apply-templates select="name" />}\vspace{5pt}\item[\textsl{<xsl:apply-templates select="name" />(<xsl:for-each select="in[1]"><xsl:if test="@opt">[</xsl:if><xsl:apply-templates select="name" /><xsl:if test="@opt">]</xsl:if></xsl:for-each><xsl:for-each select="in[position() > 1]">, <xsl:if test="@opt">[</xsl:if><xsl:apply-templates select="name" /><xsl:if test="@opt">]</xsl:if></xsl:for-each>)}]\hfil\vspace{3pt}\\
<xsl:apply-templates select="desc" />
<xsl:if test="in|out">\begin{description}
\resetwidth
<xsl:for-each select="in">\applywidth{\textsl{<xsl:apply-templates select="name" /> : <xsl:apply-templates select="type" />}---}
</xsl:for-each><xsl:for-each select="out">\applywidth{\textsl{<xsl:apply-templates select="name" /> : <xsl:apply-templates select="type" />}---}
</xsl:for-each>
<xsl:if test="in">\item[\rm Parameters:]\hfil
\begin{varlist}
<xsl:for-each select="in">\item[\textsl{<xsl:apply-templates select="name" /> : <xsl:apply-templates select="type" />}] <xsl:apply-templates select="desc" />
</xsl:for-each>\end{varlist}</xsl:if>
<xsl:if test="in and out">\vspace{1ex}</xsl:if>
<xsl:if test="out">\item[\rm Returns:]\hfil
\begin{varlist}
<xsl:for-each select="out">\item[\textsl{<xsl:apply-templates select="type" />}] <xsl:apply-templates select="desc" />
</xsl:for-each>\end{varlist}</xsl:if>\end{description}
</xsl:if>
</xsl:template>

<!--*****************************************************-->
<!--	Characters					 -->
<!--*****************************************************-->

    <xsl:template match="char[@name='amp']">\&amp;</xsl:template>
    <xsl:template match="char[@name='lt']">$&lt;$</xsl:template>
    <xsl:template match="char[@name='gt']">$&gt;$</xsl:template>
    <xsl:template match="char[@name='lbr']">\{</xsl:template>
    <xsl:template match="char[@name='rbr']">\}</xsl:template>
    <xsl:template match="char[@name='pct']">\%</xsl:template>
    <xsl:template match="char[@name='dollar']">\$</xsl:template>
    <xsl:template match="char[@name='lsquo']">`</xsl:template>
    <xsl:template match="char[@name='rsquo']">'</xsl:template>
    <xsl:template match="char[@name='ldquo']">``</xsl:template>
    <xsl:template match="char[@name='rdquo']">''</xsl:template>
    <xsl:template match="char[@name='doubledash']">-{}-</xsl:template>
    <xsl:template match="char[@name='uscore']">\_</xsl:template>
    <xsl:template match="char[@name='bslash']">$\backslash$</xsl:template>
    <xsl:template match="char[@name='tilde']">$\sim$</xsl:template>
    <xsl:template match="char[@name='hash']">\#</xsl:template>
    <xsl:template match="char[@name='bar']">$|$</xsl:template>
    <xsl:template match="char[@name='hat']">\textasciicircum{}</xsl:template>
</xsl:stylesheet>
