%option yylineno
%option stack

INLINE	"url"|"q"|"e"|"char"|"warn"|"fixme"|"tt"|"note"|"ob"|"b"|"ref"|"manpage"
POINT	"document"|"part"|"chapter"|"section"|"subsection"|"module"|"appendix"

LF		\012
CR		\015
ID		[A-Za-z0-9_](-?[A-Za-z0-9_])*
SPACE	[ \t\012\015]{1,}
QUOTED	"\""[^\"]*"\""|"'"[^\']*"'"
ATTRIB	{ID}(":"{ID})?{SPACE}?"="{SPACE}?{QUOTED}{SPACE}?

/* Warning: Don't alter the order here without
   updating the defines in parser.y */
%s LUA
%s CCODE
%s TABLE
%s VERB
%s DOC
%s SHELL

%s LUA_LCOMMENT
%s LUA_BCOMMENT
%s LUA_BSTRING
%s LUA_SSTRING
%s LUA_DSTRING
%x ESCAPE_NEXT
%x WAIT
%x DROP

%{

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <stdarg.h>

#include "parser.tab.h"
#include "general.h"

#define CRCHAR '\015'

void retstr()
{
	char *out = malloc((yyleng+1)*sizeof(char));
	strcpy(out,yytext);
	yylval.str = out;
}

void retelt()
{
	char *out = malloc((yyleng+1)*sizeof(char));
	int i = 0;
	int j = 0;
	while(yytext[j]) {
		if(yytext[j] == CRCHAR) ++j;
		else out[i++] = yytext[j++];
	}
	out[i] = 0;
	yylval.str = out;
}

void set_state(int s)
{
	BEGIN(s);
}

static void yy_pop_state();

void pop_state()
{
	yy_pop_state();
}

%}

%%

<WAIT>.|\n {
	yyless(0);
	return TOK_WAIT;
}

<ESCAPE_NEXT>. {
	yylval.chr = yytext[0];
	yy_pop_state();
	return TOK_OTHER;
}

\\{LF}	;

<DOC>"=fn"{SPACE} { return FLD_FN; }
<DOC>"=in"{SPACE} { return FLD_IN; }
<DOC>"=inopt"{SPACE} { return FLD_INOPT; }
<DOC>"=out"{SPACE} { return FLD_OUT; }
<DOC>"=d"{SPACE} { return FLD_D; }

<LUA_LCOMMENT>{LF}{LF}*	{
	yyless(0);
	yy_pop_state();
}

<LUA_BCOMMENT,LUA_BSTRING>"]]" {
	yy_pop_state();
	return KEY_BEND;
}

<LUA_SSTRING>"\\"/"'" {
	yy_push_state(ESCAPE_NEXT);
	return TOK_BSLASH;
}
<LUA_SSTRING>"'" {
	yy_pop_state();
	return TOK_APOS;
}

<LUA_DSTRING>"\\"/"\"" {
	yy_push_state(ESCAPE_NEXT);
	return TOK_BSLASH;
}
<LUA_DSTRING>"\"" {
	yy_pop_state();
	return TOK_QUOT;
}

<LUA>"\\+"		return TOK_INCTAB;
<LUA>"\\-"		return TOK_DECTAB;
<LUA>"\\!"		return TOK_IGNORE;
<LUA>"\\."		return TOK_MISSING;
<LUA>"\\*"		return TOK_SKIPLINE;
<LUA>"\\="		return TOK_RESULT;
<LUA>"\\,"		return TOK_NOTHING;

<LUA>"["	{ yy_push_state(YY_START);		return OP_LSQB;		}
<LUA,SHELL>"]"	{ yy_pop_state();				return OP_RSQB;		}
<LUA>"--[["	{ yy_push_state(LUA_BCOMMENT);	return KEY_BCOMMENT;	}
<LUA>"--"	{ yy_push_state(LUA_LCOMMENT);	return KEY_LCOMMENT;	}
<LUA>"#"	{ yy_push_state(LUA_LCOMMENT);	return TOK_HASH;	}
<LUA>"\""	{ yy_push_state(LUA_DSTRING);	return TOK_QUOT;		}
<LUA>"'"	{ yy_push_state(LUA_SSTRING);	return TOK_APOS;		}
<LUA>"[["	{ yy_push_state(LUA_BSTRING);	return KEY_BSTRING;		}

<LUA>"and"		return KEY_AND;
<LUA>"break"	return KEY_BREAK;
<LUA>"do"		return KEY_DO;
<LUA>"else"		return KEY_ELSE;
<LUA>"elseif"	return KEY_ELSEIF;
<LUA>"end"		return KEY_END;
<LUA>"false"	return KEY_FALSE;
<LUA>"for"		return KEY_FOR;
<LUA>"function"	return KEY_FUNCTION;
<LUA>"if"		return KEY_IF;
<LUA>"in"		return KEY_IN;
<LUA>"local"	return KEY_LOCAL;
<LUA>"nil"		return KEY_NIL;
<LUA>"not"		return KEY_NOT;
<LUA>"or"		return KEY_OR;
<LUA>"repeat"	return KEY_REPEAT;
<LUA>"return"	return KEY_RETURN;
<LUA>"then"		return KEY_THEN;
<LUA>"true"		return KEY_TRUE;
<LUA>"until"	return KEY_UNTIL;
<LUA>"while"	return KEY_WHILE;
<LUA>"+"		return OP_PLUS;
<LUA>"-"		return OP_MINUS;
<LUA>"*"		return OP_STAR;
<LUA>"/"		return OP_SLASH;
<LUA>"^"		return OP_HAT;
<LUA>"~="		return OP_NE;
<LUA>"<="		return OP_LE;
<LUA>">="		return OP_GE;
<LUA>"<"		return OP_LT;
<LUA>">"		return OP_GT;
<LUA>"=="		return OP_EQ;
<LUA>"="		return OP_ASSIGN;
<LUA>"("		return OP_LPAR;
<LUA>")"		return OP_RPAR;
<LUA>"{"		return OP_LBR;
<LUA>"}"		return OP_RBR;
<LUA>";"		return OP_SEMI;
<LUA>":"		return OP_COLON;
<LUA>","		return OP_COMMA;
<LUA>"..."		return OP_3DOTS;
<LUA>".."		return OP_2DOTS;
<LUA>"."		return OP_DOT;

<LUA>[A-Za-z_][A-Za-z0-9_]* { retstr(); return TOK_ID; }
<LUA>-?[0-9][0-9]*(\.[0-9][0-9]*)?([Ee]-?[0-9][0-9]*)? { retstr(); return TOK_NUM; }

<VERB,SHELL>"["		{ yy_push_state(YY_START);		return TOK_LSQB;		}
<VERB,DOC,TABLE,DROP>"]"	{ yy_pop_state();			return TOK_RSQB;		}

<INITIAL,DOC,TABLE>\\.	{
	yylval.chr = yytext[1];
	return TOK_OTHER;
}

<INITIAL,DOC,TABLE>"[["		{ yy_push_state(DROP); yy_push_state(WAIT); return MODE_DEFB; }
<INITIAL,DOC,TABLE>"["		{ yy_push_state(WAIT); return MODE_DEFI; }
<INITIAL,DOC,TABLE>"lua[["	{ yy_push_state(DROP); yy_push_state(WAIT); return MODE_LUAB; }
<INITIAL,DOC,TABLE>"lua["	{ yy_push_state(WAIT); return MODE_LUAI; }
<INITIAL,DOC,TABLE>"shell[[" { yy_push_state(DROP); yy_push_state(WAIT); return MODE_SHELLB; }
<INITIAL,DOC,TABLE>"shell["	{ yy_push_state(WAIT); return MODE_SHELLI; }
<INITIAL,DOC,TABLE>"tt["	{ yy_push_state(WAIT); return MODE_TT; }
<INITIAL,DOC,TABLE>"g["		{ yy_push_state(WAIT); return MODE_G; }
<INITIAL,DOC,TABLE>"path["	{ yy_push_state(WAIT); return MODE_PATH; }
<INITIAL,DOC,TABLE>"url["	{ yy_push_state(WAIT); return MODE_URL; }
<INITIAL>"doc[["			{ yy_push_state(DROP); yy_push_state(WAIT); return MODE_DOC; }
<INITIAL,DOC>"defs[["		{ yy_push_state(DROP); yy_push_state(WAIT); return MODE_DEFS; }
<INITIAL,DOC,TABLE>{ID}"["	{ die("Unknown mode %s",yytext); }

<INITIAL,DOC,TABLE>"<"{SPACE}?{INLINE}({SPACE}{ATTRIB}*)?">" {
	retelt();
	return TOK_ISTARTTAG;
}

<INITIAL,DOC,TABLE>"<"{SPACE}?{POINT}({SPACE}{ATTRIB}*)?">" {
	retelt();
	return TOK_PSTARTTAG;
}

<INITIAL,DOC,TABLE>"<"{SPACE}?{ID}(":"{ID})?({SPACE}{ATTRIB}*)?">" {
	retelt();
	return TOK_STARTTAG;
}

<INITIAL,DOC,TABLE>"</"{SPACE}?{ID}(":"{ID})?">" {
	retelt();
	return TOK_ENDTAG;
}

<INITIAL,DOC,TABLE>"<"{SPACE}?{INLINE}({SPACE}{ATTRIB}*)?"/>" {
	retelt();
	return TOK_IBOTHTAG;
}

<INITIAL,DOC,TABLE>"<"{SPACE}?{ID}(":"{ID})?({SPACE}{ATTRIB}*)?"/>" {
	retelt();
	return TOK_BOTHTAG;
}

<INITIAL,DOC,TABLE>"<"{SPACE}?{INLINE}({SPACE}{ATTRIB}*)?"/" {
	yytext[yyleng-1] = '>';
	retelt();
	return TOK_ISTARTSHORT;
}

<INITIAL,DOC,TABLE>"<"{SPACE}?{POINT}({SPACE}{ATTRIB}*)?"/" {
	yytext[yyleng-1] = '>';
	retelt();
	return TOK_PSTARTSHORT;
}

<INITIAL,DOC,TABLE>"<"{SPACE}?{ID}(":"{ID})?({SPACE}{ATTRIB}*)?"/" {
	yytext[yyleng-1] = '>';
	retelt();
	return TOK_STARTSHORT;
}

<INITIAL,DOC,TABLE>"<!--"[^-]*("--"[^>][^-]*|"-"[^-][^-]*)*"-->" {
	retelt();
	return TOK_XMLCOMMENT;
}

"&"			return TOK_AMP;
"<"			return TOK_LT;
">"			return TOK_GT;
"{"			return TOK_LBR;
"}"			return TOK_RBR;
"["			return TOK_LSQB;
"]"			return TOK_RSQB;
"%"			return TOK_PCT;
"$"			return TOK_DOLLAR;
"\""		return TOK_QUOT;
"\'"		return TOK_APOS;
":"			return TOK_COLON;
"|"			return TOK_BAR;
"="			return TOK_EQUALS;
"\\"		return TOK_BSLASH;
"/"			return TOK_SLASH;
";"			return TOK_SEMI;
"#"			return TOK_HASH;
"!"			return TOK_PLING;
"-"			return TOK_DASH;
"~"			return TOK_TILDE;
"^"			return TOK_HAT;

{LF}{2,}	{
	retstr();
	return TOK_PARBREAK;
}

{LF}		return TOK_LINEBREAK;
" "			return TOK_SPACE;
{CR}		;
\t			return TOK_TAB;

{ID}	{ retstr(); return TOK_ID; }

.		{
	yylval.chr = yytext[0];
	//printf("TOK_OTHER: %d '%s' State %d\n",(int) yytext[0],yytext,YY_START);
	return TOK_OTHER;
}

%%
