/***********************************************************
 * C Code
 **********************************************************/

%{
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>

#include "general.h"

#define codeprintf codef
#define codeescape codeesc
#define codeputchar codechar

enum {
	LUA = 1,
	CCODE,
	TABLE,
	VERB,
	DOC,
	SHELL
};

int yylex();
extern FILE *yyin;
void pop_state();
void set_state(int);

int linestart;
int hadbreak;
int tabdepth;
int isinline;
int tabdepth;
char *interline;
int ignorenexterror;
char *shortend;
int iscode;

void yyerror(const char *s);

void maybelinestart()
{
	if(!iscode) return;
	if(isinline) return;
	if(linestart) {
		if(hadbreak) {
			hadbreak = 0;
			printf("<codeline empty=\"yes\" />\n");
		}
		printf("<codeline depth=\"%d\">",tabdepth);
		if(interline) printf("<%s>",interline);
		linestart = 0;
	}
}

void maybeskiplinestart()
{
	if(isinline) return;
	if(linestart) {
		if(hadbreak) {
			hadbreak = 0;
			printf("<codeline empty=\"yes\" />\n");
		}
		printf("<codeline ignore=\"yes\">");
		linestart = 0;
	}
}

void escape(char *n)
{
	printf("<char name=\"%s\"/>",n);
}

void codeesc(char *n)
{
	maybelinestart();
	printf("<char name=\"%s\"/>",n);
}

void keyword(char *n)
{
	maybelinestart();
	printf("<keyword>%s</keyword>",n);
}

void codesetup()
{
	linestart = 1;
	hadbreak = 0;
	tabdepth = 0;
	ignorenexterror = 0;
	iscode = 1;
}

void codef(const char *f, ...)
{
	va_list ap;
	va_start(ap,f);
	maybelinestart();
	vprintf(f,ap);
	va_end(ap);
}

void codechar(char c)
{
	maybelinestart();
	putchar(c);
}

void escus(const char *c)
{
	for(;*c;++c) {
		if(*c == '_') codeescape("uscore");
		else codeputchar(*c);
	}
}

void parseshort(const char *t)
{
	int start = 0;
    while(!isalnum(t[start]) && t[start] != '_') ++start;
	int end = start;
	while(isalnum(t[end]) || t[end] == '_' || t[end] == '-' || t[end] == ':') ++end;

	//fprintf(stderr,"[%s] %d %d\n",t,start,end);
	shortend = malloc(sizeof(char) * (end-start+1));
    strncpy(shortend,t+start,end-start);
	shortend[end-start] = 0;
}

void bbreak(int par)
{
	if(!linestart) {
		if(interline) printf("</%s>",interline);
		printf("</codeline>\n");
		if(par) {
			printf("<codeline empty=\"yes\"/>\n");
			hadbreak = 1;
		}
		linestart = 1;
	}
}
%}

/***********************************************************
 * Declarations
 **********************************************************/

%union {
	char *str;
	char chr;
};

%token TOK_INCTAB
%token TOK_DECTAB
%token TOK_IGNORE
%token TOK_MISSING
%token TOK_SKIPLINE
%token TOK_RESULT
%token TOK_NOTHING

%token OP_LSQB
%token OP_RSQB
%token KEY_BCOMMENT
%token KEY_LCOMMENT
%token KEY_BSTRING
%token KEY_BEND

%token KEY_AND
%token KEY_BREAK
%token KEY_DO
%token KEY_ELSE
%token KEY_ELSEIF
%token KEY_END
%token KEY_FALSE
%token KEY_FOR
%token KEY_FUNCTION
%token KEY_IF
%token KEY_IN
%token KEY_LOCAL
%token KEY_NIL
%token KEY_NOT
%token KEY_OR
%token KEY_REPEAT
%token KEY_RETURN
%token KEY_THEN
%token KEY_TRUE
%token KEY_UNTIL
%token KEY_WHILE
%left OP_PLUS
%left OP_MINUS
%left OP_STAR
%left OP_SLASH
%left OP_HAT
%left OP_NE
%left OP_LE
%left OP_GE
%left OP_LT
%left OP_GT
%left OP_EQ
%token OP_ASSIGN
%token OP_LPAR
%token OP_RPAR
%token OP_LBR
%token OP_RBR
%token OP_SEMI
%token OP_COLON
%token OP_COMMA
%token OP_3DOTS
%token OP_2DOTS
%token OP_DOT

%token FLD_FN
%token FLD_IN
%token FLD_INOPT
%token FLD_OUT
%token FLD_D

%token TOK_AMP
%token TOK_LT
%token TOK_GT
%token TOK_LBR
%token TOK_RBR
%token TOK_LSQB
%token TOK_RSQB
%token TOK_PCT
%token TOK_DOLLAR
%token TOK_QUOT
%token TOK_APOS
%token TOK_COLON
%token TOK_BAR
%token TOK_EQUALS
%token TOK_BSLASH
%token TOK_SLASH
%token TOK_SEMI
%token TOK_HASH
%token TOK_PLING
%token TOK_DASH
%token TOK_TILDE
%token TOK_HAT

%token <str> TOK_PARBREAK
%token TOK_LINEBREAK
%token TOK_SPACE
%token TOK_TAB

%token <str> TOK_ID
%token <str> TOK_NUM
%token <str> TOK_STARTTAG
%token <str> TOK_ISTARTTAG
%token <str> TOK_PSTARTTAG
%token <str> TOK_STARTSHORT
%token <str> TOK_ISTARTSHORT
%token <str> TOK_PSTARTSHORT
%token <str> TOK_ENDTAG
%token <str> TOK_BOTHTAG
%token <str> TOK_IBOTHTAG
%token <str> TOK_XMLCOMMENT
%token <chr> TOK_OTHER
%token TOK_WAIT

%token MODE_DEFB
%token MODE_DEFI
%token MODE_DEFS
%token MODE_LUAB
%token MODE_LUAI
%token MODE_SHELLB
%token MODE_SHELLI
%token MODE_TT
%token MODE_G
%token MODE_PATH
%token MODE_URL
%token MODE_DOC

%%

/***********************************************************
 * Main Grammar
 **********************************************************/

input:			{ printf("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<xfile xmlns:xi=\"http://www.w3.org/2001/XInclude\">\n"); } optignws body { printf("</xfile>\n"); }
		;

eatwaits:		// nothing
		|		TOK_WAIT { warn("Waited"); } eatwaits
		;

body:			// nothing
		|		paragraph optignws body
		|		blockelt optignws body
		;

bodyns:			// nothing
		|		paragraphns optignws bodyns
		;

paragraph:		startbodytext optparbreak { printf("</para>\n\n"); }
		;

paragraphns:	startbodytextns optparbreak { printf("</para>\n\n"); }
		;

startbodytext:	{ printf("<para>"); } bodytext
		;

startbodytextns:	{ printf("<para>"); } bodytextns
		;

bodytext:		bodytext0
		|		bodytext0 bodytext
		;

bodytextns:		bodytextns0
		|		bodytextns0 bodytextns
		;

bodytextnb:		bodytextnb0
		|		bodytextnb0 bodytextnb
		;

bodytext0:		inlineelt
		|		normallit
		|		tok_quot
		|		tok_apos
		|		usuallit
		|		bodyws
		;

bodytextns0:	inlineelt
		|		normallit
		|		tok_quot
		|		tok_apos
		|		bodyws
		|		tok_pling
		|		tok_dash
		|		tok_gt { warn("Lonely >"); }
		|		tok_amp
		|		tok_bar
		;

bodytextnb0:	inlineelt
		|		normallit
		|		tok_quot
		|		tok_apos
		|		tok_pling
		|		tok_dash
		|		tok_gt { warn("Lonely >"); }
		|		tok_amp
		|		tok_slash
		|		bodyws
		;

optignws:		// nothing
		|		ignws
		;

ignws:			TOK_PARBREAK { free($1); } optignws
		|		TOK_LINEBREAK optignws
		|		TOK_SPACE optignws
		|		TOK_TAB optignws
		;

/*optignwsp:		// nothing
		|		ignwsp
		;

ignwsp:			TOK_LINEBREAK optignws
		|		TOK_SPACE optignws
		|		TOK_TAB optignws
		;
*/

optbodyws:		// nothing
		|		bodyws
		;

bodyws:			TOK_LINEBREAK { putchar('\n'); } optignws
		|		TOK_SPACE { putchar(' '); } optbodyws
		|		TOK_TAB { putchar('\t'); } optbodyws
		;

optparbreak:	// nothing
		|		parbreak
		;

parbreak: 		TOK_PARBREAK { free($1); } optignws
		;

optlitws:		// nothing
		|		litws
		;

litws:			TOK_PARBREAK { printf("%s",$1); free($1); } optlitws
		|		TOK_LINEBREAK { putchar('\n'); } optlitws
		|		TOK_SPACE { putchar(' '); } optlitws
		|		TOK_TAB { putchar('\t'); } optlitws
		;

optlitls:		// nothing
		|		litls
		;

litls:			TOK_SPACE { putchar(' '); } optlitls
		|		TOK_TAB { putchar('\t'); } optlitls
		;

optcodews:		// nothing
		|		codews
		;

codews:			TOK_PARBREAK { if(!linestart) { printf("</codeline>\n"); hadbreak = 1; linestart = 1; } } optignws
		|		TOK_LINEBREAK { if(!linestart) printf("</codeline>\n"); linestart = 1; } optignws
		|		TOK_SPACE { putchar(' '); } optignws
		|		TOK_TAB { putchar(' '); } optignws
		;

blockelt:		optignws TOK_STARTTAG { printf("%s",$2);free($2); } optignws body endtag { printf("\n\n"); }
		|		optignws TOK_PSTARTTAG { printf("%s",$2); free($2); } optignws bodytext endtag { printf("\n\n"); }
		|		optignws TOK_STARTSHORT { printf("%s",$2); parseshort($2); free($2); } optignws bodyns endshort { printf("\n\n"); }
		|		optignws TOK_PSTARTSHORT { printf("%s",$2); parseshort($2); free($2); } optignws bodytextns endshort { printf("\n\n"); }
		|		xmlcomment
		|		TOK_BOTHTAG { codeprintf("%s",$1); free($1); }
		|		newblockmode
		;

inlineelt:		TOK_ISTARTTAG { printf("%s",$1); free($1); } optignws bodytext endtag
		|		TOK_ISTARTSHORT { printf("%s",$1); parseshort($1); free($1); } optignws bodytextns endshort
		|		TOK_IBOTHTAG { printf("%s",$1); free($1); }
		|		xmlcomment
		|		newinlinemode
		;

endtag:			TOK_ENDTAG { printf("%s",$1); free($1); }
		;

endshort:		TOK_SLASH { printf("</%s>",shortend); free(shortend); }
		;

xmlcomment:		TOK_XMLCOMMENT { printf("%s",$1); free($1); }
		;

normallit:		tok_lbr
		|		tok_rbr
		|		tok_pct
		|		tok_dollar
		|		tok_colon
		|		tok_equals
		|		tok_bslash
		|		tok_semi
		|		tok_hash
		|		tok_tilde
		|		tok_hat
		|		tok_id
		|		tok_other
		;

tok_lbr:		TOK_LBR		{ codeescape("lbr"); } ;
tok_rbr:		TOK_RBR		{ codeescape("rbr"); } ;
tok_pct:		TOK_PCT 	{ codeescape("pct"); } ;
tok_dollar:		TOK_DOLLAR	{ codeescape("dollar"); } ;
tok_colon:		TOK_COLON	{ codeputchar(':'); } ;
tok_bar:		TOK_BAR		{ codeescape("bar"); } ;
tok_equals:		TOK_EQUALS	{ codeputchar('='); } ;
tok_bslash:		TOK_BSLASH	{ codeescape("bslash"); } ;
tok_slash:		TOK_SLASH	{ codeputchar('/'); } ;
tok_semi:		TOK_SEMI	{ codeputchar(';'); } ;
tok_hash:		TOK_HASH    { codeescape("hash"); } ;
tok_tilde:		TOK_TILDE   { codeescape("tilde"); } ;
tok_hat:		TOK_HAT		{ codeescape("hat"); } ;
tok_id:			TOK_ID		{ escus($1); free($1); } ;
tok_other:		TOK_OTHER	{ codeputchar($1); } ;

usuallit:		tok_pling
		|		tok_dash
		|		tok_lt
		|		tok_gt
		|		tok_amp
		|		tok_slash
		|		tok_bar
		;

tok_apos:		TOK_APOS	{ codeputchar('\''); } ;
tok_quot:		TOK_QUOT	{ codeputchar('\"'); } ;
tok_amp:		TOK_AMP		{ codeescape("amp"); } ;
tok_gt:			TOK_GT		{ codeescape("gt"); } ;
tok_lt:			TOK_LT		{ codeescape("lt"); } ;
tok_pling:		TOK_PLING	{ codeputchar('!'); } ;
tok_dash:		TOK_DASH	{ codeputchar('-'); } ;

dlit:			optlitws dlit0
		;

dlit0:			// nothing
		| 		normallit dlit
		|		tok_apos dlit
		|		tok_pling dlit
		|		tok_dash dlit
		|		tok_lt dlit
		|		tok_gt dlit
		|		tok_amp dlit
		|		tok_slash dlit
		|		tok_bar dlit
		|		litmode dlit
		;

slit:			optlitws slit0
		;

slit0:			// nothing
		|		normallit slit
		|		tok_quot slit
		|		usuallit slit
		|		litmode slit
		;

llit:			optlitls llit0
		;

llit0:			// nothing
		|		normallit llit
		|		tok_quot llit
		|		tok_apos llit
		|		tok_amp llit
		| 		tok_pling llit
		|		tok_dash llit
		|		tok_slash llit
		|		litmode llit
		|		tok_lt llit
		|		tok_gt llit
		|		tok_bar
		;

blit:			optlitls blit0
		|		TOK_PARBREAK { free($1); bbreak(1); } blit
		|		TOK_LINEBREAK { bbreak(0); } blit
		;

blit0:			// nothing
		|		normallit blit
		|		tok_quot blit
		|		tok_apos blit
		|		tok_amp blit
		| 		tok_pling blit
		|		tok_dash blit
		|		tok_slash blit
		|		litmode blit
		|		tok_lt blit
		|		tok_gt blit
		|		tok_bar blit
		;

litnonws:		// nothing
		| 		normallit litnonws
		|		tok_quot litnonws
		|		tok_apos litnonws
		|		tok_amp litnonws
		| 		tok_pling litnonws
		|		tok_dash litnonws
		|		tok_slash litnonws
		|		litmode litnonws
		|		tok_lt litnonws
		|		tok_gt litnonws
		|		tok_bar
		;

litmode:		TOK_LSQB	{ codeputchar('['); }
		|		TOK_RSQB	{ codeputchar(']'); }
		|		MODE_DEFB	{ codeprintf("[["); pop_state(); pop_state(); } eatwaits
		|		MODE_DEFI	{ codeprintf("["); pop_state(); } eatwaits
		|		MODE_LUAB	{ codeprintf("lua[["); pop_state(); pop_state(); } eatwaits
		|		MODE_LUAI	{ codeprintf("lua["); pop_state(); } eatwaits
		|		MODE_SHELLB	{ codeprintf("shell[["); pop_state(); pop_state(); } eatwaits
		|		MODE_SHELLI	{ codeprintf("shell["); pop_state(); } eatwaits
		|		MODE_TT		{ codeprintf("tt["); pop_state(); } eatwaits
		|		MODE_G		{ codeprintf("g["); pop_state(); } eatwaits
		|		MODE_PATH	{ codeprintf("path["); pop_state(); } eatwaits
		|		MODE_URL	{ codeprintf("url["); pop_state(); } eatwaits
		|		MODE_DOC	{ codeprintf("doc[["); pop_state(); pop_state(); } eatwaits
		|		MODE_DEFS	{ codeprintf("defs[["); pop_state(); pop_state(); } eatwaits
		;

newblockmode:	MODE_DEFB	{ set_state(LUA); } eatwaits lua_mainb TOK_RSQB
		|		MODE_LUAB	{ set_state(LUA); } eatwaits lua_mainb TOK_RSQB
		|		MODE_SHELLB	{ set_state(SHELL); } eatwaits shell_mainb TOK_RSQB
		|		MODE_DOC	{ set_state(DOC); } eatwaits doc_main TOK_RSQB
		|		MODE_DEFS	{ set_state(TABLE); } eatwaits defs_main TOK_RSQB
		;

newinlinemode:	MODE_DEFI	{ set_state(LUA); } eatwaits lua_maini OP_RSQB
		|		MODE_LUAI	{ set_state(LUA); } eatwaits lua_maini OP_RSQB
		|		MODE_SHELLI	{ set_state(SHELL); } eatwaits lua_maini OP_RSQB
		|		MODE_TT		{ set_state(VERB); } eatwaits tt_main TOK_RSQB
		|		MODE_G		{ set_state(VERB); } eatwaits g_main TOK_RSQB
		|		MODE_PATH	{ set_state(VERB); } eatwaits path_main TOK_RSQB
		|		MODE_URL	{ set_state(VERB); } eatwaits url_main TOK_RSQB
		;

tt_main:		{ printf("<tt>"); } bodytext { printf("</tt>"); }
		;

g_main:			{ printf("<code style=\"g\">"); } bodytext { printf("</code>"); }
		;

url_main:		{ printf("<url>"); } url_spec { printf("</url>"); }
		;

path_main:		{ printf("<path>"); } url_spec { printf("</path>"); }
		;

url_spec:		TOK_SLASH { codeprintf("/"); } url_spec
		|		url_body
		;

url_body:		// nothing
		|		bodytextns
		|		bodytextns url_slashes { printf("<ob/>"); } url_body
		;

url_slashes:	TOK_SLASH { codeprintf("/"); }
		|		TOK_SLASH { codeprintf("/"); } url_slashes
		;

shell_mainb:	{ printf("<codeblock style=\"shell\">"); codesetup(); isinline = 0; } shell_disp OP_RSQB { if(!linestart) printf("</codeline>"); printf("</codeblock>\n\n"); iscode = 0; }
		;

shell_maini:	{ printf("<code style=\"shell\">"); codesetup(); isinline = 1; } shell_disp { printf("</code>"); iscode = 0; }
		;

shell_disp:		blit
		;

/***********************************************************
 * Lua Grammar
 **********************************************************/

lua_mainb:		{ printf("<codeblock style=\"lua\">"); codesetup(); isinline = 0; } lua_disp OP_RSQB { if(!linestart) printf("</codeline>"); printf("</codeblock>\n\n"); iscode = 0; }
		;

lua_maini:		{ printf("<code style=\"lua\">"); codesetup(); isinline = 1; } lua_disp { printf("</code>"); iscode = 0; }
		;

lua_disp:		lua_ws lua_chunk
		|		lua_ws lua_exp
		|		error
		;

lua_chunk: 		// nothing
		|		lua_stat lua_chunk
		|		lua_stat OP_SEMI { codechar(';'); } lua_ws lua_chunk
		|		lua_return { --tabdepth; }
		| 		lua_return lua_explist { --tabdepth; }
		;

lua_block: 		lua_chunk
		;

lua_stat: 		lua_varlist lua_assign lua_explist { --tabdepth; }
		| 		lua_functioncall
		| 		lua_do lua_block lua_end
		| 		KEY_WHILE { keyword("while"); ++tabdepth; } lua_ws lua_exp lua_do_dec lua_block lua_end
		| 		KEY_REPEAT { keyword("repeat"); } lua_ws lua_block KEY_UNTIL { keyword("until"); } lua_ws lua_exp
		| 		lua_if lua_exp lua_then lua_block lua_stat_if lua_end
		| 		lua_if lua_exp lua_then lua_block lua_stat_if KEY_ELSE { --tabdepth; keyword("else"); ++tabdepth; } lua_ws lua_block lua_end
		| 		KEY_BREAK { keyword("break"); } lua_ws
		| 		lua_for lua_name lua_assign lua_exp lua_comma lua_exp { --tabdepth; } lua_do_dec lua_block lua_end 
		| 		lua_for lua_name lua_assign lua_exp lua_comma lua_exp lua_comma lua_exp { --tabdepth; } lua_do_dec lua_block lua_end 
		| 		lua_for lua_name lua_stat_for KEY_IN { --tabdepth; keyword("in"); ++tabdepth; } lua_ws lua_explist lua_do_dec lua_block lua_end 
		| 		lua_func lua_funcname lua_funcbody
		| 		lua_local lua_func lua_name lua_funcbody
		| 		lua_local lua_namelist
		| 		lua_local lua_namelist lua_init
		|		lua_missing
		;

lua_stat_if:	// nothing
		|		KEY_ELSEIF { --tabdepth; keyword("elseif"); ++tabdepth; } lua_ws lua_exp lua_then lua_block lua_stat_if
		;

lua_stat_for:	// nothing
		|		lua_comma lua_name lua_stat_for
		;

lua_funcname:	lua_name lua_funcname0
		|		lua_name lua_funcname0 OP_COLON { codechar(':'); } lua_ws lua_name
		;

lua_funcname0:	// nothing
		|		OP_DOT { codechar('.'); } lua_ws lua_name lua_funcname0
		;

lua_varlist:	lua_var lua_varlist0
		;

lua_varlist0:	// nothing
		|		lua_comma lua_var lua_varlist0
		;

lua_var:		lua_name
		|		lua_prefixexp OP_LSQB { codechar('['); ++tabdepth; } lua_ws lua_exp OP_RSQB { codechar(']'); --tabdepth; } lua_ws
		|		lua_prefixexp OP_DOT { codechar('.'); } lua_ws lua_name
		;

lua_namelist:	lua_name lua_namelist0
		;

lua_namelist0:	// nothing
		|		lua_comma lua_name lua_namelist0
		;

lua_init:		lua_assign lua_explist { --tabdepth; }
		;

lua_explist:	// nothing
		|		lua_explist0
		;

lua_explist0:	lua_exp
		|		lua_exp lua_comma lua_explist0
		;

lua_exp:		KEY_NIL { keyword("nil"); } lua_ws
		|		KEY_FALSE { keyword("false"); } lua_ws
		|		KEY_TRUE { keyword("true"); } lua_ws
		|		lua_number
		|		lua_literal
		|		lua_function
		|		lua_prefixexp
		|		lua_tableconstructor
		|		lua_exp lua_binop lua_exp
		|		lua_unop lua_exp
		|		lua_missing
		;

lua_prefixexp:	lua_var
		|		lua_functioncall
		|		lua_lpar lua_exp lua_rpar
		;

lua_functioncall:
				lua_prefixexp lua_args
		|		lua_prefixexp OP_COLON { codechar(':'); } lua_ws lua_name lua_args
		;

lua_args:		lua_lpar lua_explist lua_rpar
		|		lua_tableconstructor
		|		lua_literal
		;

lua_function:	lua_func lua_funcbody
		;

lua_funcbody:	lua_lpar lua_parlist lua_rpar lua_block lua_end
		;

lua_parlist:	// nothing
		| 		lua_name lua_parlist0
		|		lua_name lua_parlist0 lua_comma OP_3DOTS { codef("..."); } lua_ws
		|		OP_3DOTS { codef("..."); } lua_ws
		;

lua_parlist0:	// nothing
		|		lua_comma lua_name lua_parlist0
		;

lua_tableconstructor:
				OP_LBR { codeesc("lbr"); ++tabdepth; } lua_ws lua_fieldlist OP_RBR { --tabdepth; codeesc("rbr"); } lua_ws
		;

lua_fieldlist:	lua_field lua_fieldlist0 lua_fieldsep
		|		lua_field lua_fieldlist0
		;

lua_fieldlist0:	// nothing
		|		lua_fieldsep lua_field lua_fieldlist0
		;

lua_field:		OP_LSQB { codechar('['); ++tabdepth; } lua_ws lua_exp OP_RSQB { --tabdepth; codechar(']'); } lua_ws lua_assign lua_exp
		|		lua_name lua_assign lua_exp { --tabdepth; }
		|		lua_exp
		;

lua_fieldsep:	lua_comma
		|		OP_SEMI		{ codechar(';'); } lua_ws
		;

lua_binop:		OP_PLUS		{ codechar('+'); } lua_ws
		|		OP_MINUS	{ codechar('-'); } lua_ws
		|		OP_STAR		{ codechar('*'); } lua_ws
		|		OP_SLASH	{ codechar('/'); } lua_ws
		|		OP_HAT		{ codechar('^'); } lua_ws
		|		OP_2DOTS	{ codef(".."); } lua_ws
		|		OP_LT		{ codeesc("lt"); } lua_ws
		|		OP_LE		{ codeesc("lt"); codechar('='); } lua_ws
		|		OP_GT		{ codeesc("gt"); } lua_ws
		|		OP_GE		{ codeesc("gt"); codechar('='); } lua_ws
		|		OP_EQ		{ codef("=="); } lua_ws
		|		OP_NE		{ codeesc("tilde"); codef("="); } lua_ws
		|		KEY_AND		{ keyword("and"); } lua_ws
		|		KEY_OR		{ keyword("or"); } lua_ws
		;

lua_unop:		OP_MINUS	{ codechar('-'); } lua_ws
		|		KEY_NOT		{ keyword("not"); } lua_ws
		;

lua_name:		TOK_ID		{ codef(""); escus($1); free($1); } lua_ws
		;

lua_number:		TOK_NUM		{ codef("%s",$1); free($1); } lua_ws
		;

lua_literal:	TOK_QUOT { codef("<string>"); codeesc("ldquo"); } dlit TOK_QUOT { codeesc("rdquo"); codef("</string>"); } lua_ws
		|		TOK_APOS { codef("<string>"); codeesc("lsquo"); } slit TOK_APOS { codeesc("rsquo"); codef("</string>"); } lua_ws
		|		KEY_BSTRING { interline="string"; codef("<string>[["); } blit KEY_BEND { codef("]]</string>"); interline = 0; } lua_ws
		;

lua_ws:			optcodews
		|		optcodews lua_comment
		|		optcodews lua_special lua_ws
		;

lua_special:	TOK_INCTAB { ++tabdepth; }
		|		TOK_DECTAB { --tabdepth; }
		|		TOK_IGNORE { ignorenexterror = 1; }
		|		TOK_SKIPLINE { maybeskiplinestart(); }
		|		TOK_NOTHING { maybelinestart(); }
		|		TOK_RESULT { interline = (linestart ? "result" : "lineresult"); if(!linestart) codef("<%s>",interline); } llit { codef("</%s>",interline); interline = 0; } lua_ws
		;

lua_missing:	TOK_MISSING { codef("<missingcode/>"); } lua_ws
		|		OP_3DOTS { codef("..."); } lua_ws
		;

lua_comment:	KEY_LCOMMENT { interline = (linestart ? "comment" : "linecomment"); codeesc("doubledash"); } llit { codef("</%s>",interline); interline = 0; } lua_ws
		|		TOK_HASH { interline = (linestart ? "comment" : "linecomment"); codeesc("hash"); } llit { codef("</%s>",interline); interline = 0; } lua_ws
		|		KEY_BCOMMENT { interline = (linestart ? "comment" : "linecomment"); codeesc("doubledash"); codef("[["); } blit KEY_BEND { codef("]]"); codef("</%s>",interline); interline = 0; } lua_ws
		;

lua_assign: 	OP_ASSIGN	{ codechar('='); ++tabdepth; } lua_ws ;
lua_comma:		OP_COMMA	{ codechar(','); } lua_ws ;
lua_lpar:		OP_LPAR		{ codechar('('); ++tabdepth; } lua_ws ;
lua_rpar:		OP_RPAR		{ --tabdepth; codechar(')'); } lua_ws ;
lua_do:			KEY_DO		{ keyword("do"); ++tabdepth; } lua_ws ;
lua_do_dec:		KEY_DO		{ --tabdepth; keyword("do"); ++tabdepth; } lua_ws ;
lua_end:		KEY_END		{ --tabdepth; keyword("end"); } lua_ws ;
lua_if:			KEY_IF		{ keyword("if"); ++tabdepth; } lua_ws ;
lua_for:		KEY_FOR		{ keyword("for"); ++tabdepth; } lua_ws ;
lua_local:		KEY_LOCAL	{ keyword("local"); } lua_ws ;
lua_return:		KEY_RETURN	{ keyword("return"); ++tabdepth; } lua_ws ;
lua_then:		KEY_THEN	{ --tabdepth; keyword("then"); ++tabdepth; } lua_ws ;
lua_func:		KEY_FUNCTION { keyword("function"); ++tabdepth; } lua_ws ;

/***********************************************************
 * Doc Grammar
 **********************************************************/

doc_main:		{ printf("<doc>\n"); } doc_body TOK_RSQB { printf("</doc>\n"); }
		;

doc_body:		optignws
		|		optignws doc_def doc_body
		|		optignws xmlcomment doc_body
		;

doc_def:		doc_fndef
		;

doc_fndef:		FLD_FN { printf("<function><name>"); } doc_entry { printf("</name>\n"); } doc_fields { printf("</function>\n"); }
		;

doc_fields:		// nothing
		|		doc_field doc_fields
		;

doc_field:		FLD_IN { printf("<in><name>"); } doc_entry { printf("</name><type>"); } doc_entry { printf("</type><desc>"); } doc_rest { printf("</desc></in>\n"); }
		|		FLD_INOPT { printf("<in opt=\"yes\"><name>"); } doc_entry { printf("</name><type>"); } doc_entry { printf("</type><desc>"); } doc_rest { printf("</desc></in>\n"); }
		|		FLD_OUT { printf("<out><type>"); } doc_entry { printf("</type><desc>"); } doc_rest { printf("</desc></out>\n"); }
		|		FLD_D { printf("<desc>"); } doc_rest { printf("</desc>\n"); }
		;

doc_entry:		litnonws ignws
		;

doc_rest:		body
		;

/***********************************************************
 * Defs Grammar
 **********************************************************/

defs_main:		{ printf("<defs>\n"); } optignws defs_defs { printf("</defs>\n"); } TOK_RSQB
		;

defs_defs:		// nothing
		|		defs_def defs_defs
		;

defs_def:		{ printf("<para><def>"); } bodytextnb TOK_BAR { printf("</def>"); } optignws bodytext optparbreak { printf("</para>\n"); }
		;

%%

/***********************************************************
 * Driver
 **********************************************************/

void yyerror(const char *s)
{
	if(ignorenexterror) {
		ignorenexterror = 0;
		return;
	}
	if(yydebug) die("%s",s);
	else warn("%s",s);
}

int main(int argc, char **argv)
{
	if(argc >= 2 && !strcmp(argv[1],"-d")) {
		yydebug = 1;
		--argc;
		++argv;
	}

	if(argc == 1) yyin = stdin;
	else if(argc == 2) yyin = fopen(argv[1],"r");
	else die("Unexpected arguments");

	if(!yyin) die("Can't open file");

	iscode = 0;
	return yyparse();
}
