assert(cheia.loadmodule("fs"))


function fs._self_test(verbose)
	
	-- see if listdir() seems to work
	local list = fs.listdir(cheia.module_paths[1])
	local found = false
	table.foreach(list, function (k,v) if v == "luacheia.lua" then found = true end end)
	assert(found, "fs.listdir(cheia.module_paths[1]) couldn't find 'luacheia.lua'")

	if verbose then print("listdir OK") end

	-- see if listdir2() seems to work
	local list = fs.listdir2(cheia.module_paths[1])
	local found = false
	table.foreach(list,
		function (k,v)
			if k == "fs.lua" and v.type == "file" then
				-- check file size against fs.fileinfo
				local info = fs.fileinfo(v.fullpath)
				if info.size == v.size then
					found = true
				end
			end
		end
	)
	assert(found, "problem with fs.listdir2() and/or fs.fileinfo()")

	if verbose then print("listdir2 OK") end

	-- exercise chdir and getcwd

	function clean_path(p)
		local clean = p

		-- replace any backslashes with slashes, so path comparisons on
		-- Windows will work
		clean = string.gsub(clean, "\\", "/")

		-- lose any "/private/automount" prefix -- this shows up on sf.net's
		-- Darwin compile-farm server and confuses our self-test.
		clean = string.gsub(clean, "^/private/automount", "")

		return clean
	end

	local original_path = fs.getcwd()
	assert(original_path)
	if verbose then print("getcwd OK") end
	fs.chdir(cheia.module_paths[1])
	assert(clean_path(fs.getcwd()) == clean_path(cheia.module_paths[1]))
	if verbose then print("chdir OK") end
	fs.chdir(original_path)
	assert(clean_path(fs.getcwd()) == clean_path(original_path))

	if verbose then print("all OK") end

	return 1
end


 -- module loaded successfully
return 1
