-- mapm.lua
-- support code for mapm library

assert(cheia.loadmodule("mapm"))
do
 local T=mapm.metatable
 T.__add=mapm.add
 T.__sub=mapm.sub
 T.__mul=mapm.mul
 T.__div=mapm.div
 T.__pow=mapm.pow
 T.__lt=function (a,b) return mapm.compare(a,b)<0 end
 T.__eq=function (a,b) return mapm.compare(a,b)==0 end
 T.__unm=function (a) return mapm.sub(0,a) end
 T.__tostring=mapm.tostring
-- T.__tostring= function (x) return mapm.tostring(x,-1) end
end

function mapm.tonumber(x) return tonumber(mapm.tostring(x,20,true)) end


--
-- test mapm library
--
function mapm._self_test(verbose)
 local my_print
 if verbose then my_print = print
 else my_print = function () end
 end

 ------------------------------------------------------------------------------
 my_print(mapm.version)
 
 ------------------------------------------------------------------------------
 my_print""
 my_print"Pi algorithm of order 4"
 
 mapm.digits(65)
 local PI=mapm.number"3.1415926535897932384626433832795028841971693993751058209749445923078164062862090"
 
 -- http://pauillac.inria.fr/algo/bsolve/constant/pi/pi.html
 local function A2()
  local x=mapm.sqrt(2)
  local p=2+x
  local y=mapm.sqrt(x)
  my_print(-1,p)
  x=(y+1/y)/2
  p=p*(x+1)/(y+1)
  my_print(0,p)
  for i=1,20 do
   local P=p
   local t=mapm.sqrt(x)
   y=(y*t+1/t)/(y+1)
   x=(t+1/t)/2
   p=p*(x+1)/(y+1)
   my_print(i,p)
   if p==P then break end
  end
  return p
 end
 
 local p=A2()
 my_print("exact",PI)
 my_print("-",mapm.abs(PI-p))

 assert(mapm.tostring(p) == "3.14159265358979323846264338327950288419716939937510582097494459229")
 
 ------------------------------------------------------------------------------
 my_print""
 my_print"Square root of 2"
 
 local function mysqrt(x)
  local y,z=x,x
  repeat z,y=y,(y+x/y)/2 until z==y
  return y
 end
 
 my_print("f math",math.sqrt(2))
 my_print("f mine",mysqrt(2))
 local a=mapm.sqrt(2) my_print("B sqrt",a)
 local b=mysqrt(mapm.number(2)) my_print("B mine",b)
 local R=mapm.number"1.414213562373095048801688724209698078569671875376948073176679737990732478462107038850387534327641573"
 my_print("exact",R)
 my_print(a==b,a<b,a>b,mapm.compare(a,b))

 assert(math.abs(math.sqrt(2) - mysqrt(2)) < 0.00001)
 assert(mapm.abs(R - a) < mapm.number"0.00000000000000000000000000000000000000000000000000000000000000001")
 assert(mapm.tostring(b) == "1.41421356237309504880168872420969807856967187537694807317667973799")
 
 ------------------------------------------------------------------------------
 my_print""
 my_print"Fibonacci numbers as digits in fraction"
 
 x=99989999
 mapm.digits(68)
 a=mapm.div(1,x)
 local s=mapm.tostring(a)
 my_print("1/"..x.." =")
 my_print("",s)
 s=string.gsub(s,"0(%d%d%d)"," %1")
 my_print("",s)

 assert(mapm.tostring(a) == "0.00000001000100020003000500080013002100340055008901440233037706100987");
 
 ------------------------------------------------------------------------------
 my_print""
 my_print"Factorials"
 
 local function factorial(n,f)
  local i
  for i=2,n do f=f*i end
  return f
 end
 
 local one=mapm.number(1)
 local i
 for i=1,30 do
  my_print(i,factorial(i,1),mapm.tostring(factorial(i,one),0))
 end

 assert(mapm.tostring(factorial(30,one),0) == "265252859812191058636308480000000")
 
 ------------------------------------------------------------------------------
 my_print""
 my_print"Comparisons"
 
 mapm.digits(4)
 a=mapm.div(4,2)
 b=mapm.number(1)
 my_print("a","b","a==b","a<b","a>b","mapm.compare(a,b)")
 my_print(a,b,a==b,a<b,a>b,mapm.compare(a,b))
 b=b+1
 my_print(a,b,a==b,a<b,a>b,mapm.compare(a,b))
 b=b+1
 my_print(a,b,a==b,a<b,a>b,mapm.compare(a,b))

 assert(not (a == b))
 assert(a < b)
 assert(not (a > b))
 assert(mapm.compare(a,b) == -1)
 
 ------------------------------------------------------------------------------
 my_print""
 my_print"Is exp(pi*sqrt(163)) an integer?"
 
 for n=1,40 do
  mapm.digits(n)
  local pi=4*mapm.atan(1)
  a=mapm.exp(pi*mapm.sqrt(163))
  my_print(n,a)
 end
 local n=40
 my_print("f",string.format("%."..n.."f",math.exp(math.pi*math.sqrt(163))))

 assert(mapm.tostring(a) == "262537412640768743.9999999999992500725968700000000000000000")
 
 ------------------------------------------------------------------------------
 my_print""
 my_print(mapm.version)

 return true
end -- mapm._self_test()


-- module successfully loaded.
return true
