-- LuaCheia wrapper for lhf's posix module.

assert(cheia.loadmodule("posix"))

-- test function
function posix._self_test(verbose)
	-- test posix library (tulrich: tests taken from lhf's original
	-- test.lua; I've stripped out some of the tests that might write
	-- to the filesystem)
	
	local d, f, g, i, k, pid	-- don't pollute globals!

	function myprint(s) if verbose then print(s) end end
	function testing(s)
	 myprint""
	 myprint("-------------------------------------------------------",s)
	end
	
	function myassert(w,c,s)
	 if not c then myprint(w,s) end
	 return c
	end
	
	function myjoin(s,...)
	 if verbose then
	  for i=1,table.getn(arg) do
	   io.write(arg[i],s)
	  end
	  io.write"\n"
	 end
	end
	
	local ox=posix

	------------------------------------------------------------------------------
	myprint(ox.version)
	
	------------------------------------------------------------------------------
	testing"uname"
	function f(x) myprint(ox.uname(x)) end
	f()
	f("Machine %n is a %m running %s %r")
	
	------------------------------------------------------------------------------
	testing"terminal"
	myprint(ox.getlogin(),ox.ttyname(),ox.ctermid(),ox.getenv"TERM")
	myprint(ox.getlogin(),ox.ttyname(1),ox.ctermid(),ox.getenv"TERM")
	myprint(ox.getlogin(),ox.ttyname(2),ox.ctermid(),ox.getenv"TERM")
	
	------------------------------------------------------------------------------
	testing"getenv"
	function f(v) myprint(v,ox.getenv(v)) end
	f"USER"
	f"HOME"
	f"SHELL"
	f"absent"
	for k in ox.getenv() do if verbose then io.write(k,"\t") end end if verbose then io.write"\n" end
	
	------------------------------------------------------------------------------
	testing"putenv"
	function f(s) myprint("putenv",s,(ox.putenv(s) or -1)>=0) end
	function g(v) myprint("now",v,ox.getenv(v)) end
	f"MYVAR=123"	g"MYVAR"
	f"MYVAR="	g"MYVAR"
	f"MYVAR"	g"MYVAR"
	
	------------------------------------------------------------------------------
	testing"getcwd, chdir, mkdir, rmdir"
	function f(d) myassert("chdir",ox.chdir(d)) g() end
	function g() local d=ox.getcwd() myprint("now at",d) return d end
--	myassert("rmdir",ox.rmdir"x")
	d=g()
	f".."
	f"xxx"
	f"/etc/uucp"
	f"/var/spool/mqueue"
	f(d)
--	assert(ox.mkdir"x")
--	f"x"
--	f"../x"
--	assert("rmdir",ox.rmdir"../x")
--	myassert("mkdir",ox.mkdir".")
--	myassert("rmdir",ox.rmdir".")
	g()
	f(d)
--	myassert("rmdir",ox.rmdir"x")
	
	------------------------------------------------------------------------------
	testing"fork, exec"
	if verbose then io.flush() end
	pid=assert(ox.fork())
	if pid==0 then
		pid=ox.getprocessid"pid"
		ppid=ox.getprocessid"ppid"
		if verbose then io.write("in child process ",pid," from ",ppid,".\nnow executing date... ") end
		if verbose then io.flush() end
--		assert(ox.exec("date","+[%c]"))
		assert(ox.exec("test", "1"))
		myprint"should not get here"
		assert(nil, "exec didn't exit!")
	else
		if verbose then io.write("process ",ox.getprocessid"pid"," forked child process ",pid,". waiting...\n") end
		ox.wait(pid)
		if verbose then io.write("child process ",pid," done\n") end
	end
	
	------------------------------------------------------------------------------
	testing"dir, stat"
	function g() local d=ox.getcwd() myprint("now at",d) return d end
	g()
	for f in ox.files"." do
	  local T=assert(ox.stat(f))
	  local p=assert(ox.getpasswd(T.uid))
	  local g=assert(ox.getgroup(T.gid))
	  myprint(T.mode,p.name.."/"..g.name,T.size,os.date("%b %d %H:%M",T.mtime),f,T.type)
	end
	
--	------------------------------------------------------------------------------
--	testing"umask"
--	ox.unlink"xxx"
--	myprint(ox.umask())
--	myprint(ox.umask("a-r,o+w"))
--	io.close(io.open("xxx","w"))
--	os.execute"ls -l xxx"
--	ox.unlink"xxx"
	
--	------------------------------------------------------------------------------
--	testing"chmod, access"
--	ox.unlink"xxx"
--	myprint(ox.access("xxx"))
--	io.close(io.open("xxx","w"))
--	myprint(ox.access("xxx"))
--	os.execute"ls -l xxx"
--	myprint(ox.access("xxx","r"))
--	assert(ox.chmod("xxx","a-rwx,o+x"))
--	myprint(ox.access("xxx","r"))
--	os.execute"ls -l xxx"
--	ox.unlink"xxx"
	
--	------------------------------------------------------------------------------
--	testing"utime"
--	io.close(io.open("xxx","w"))
--	os.execute"ls -l xxx test.lua"
--	a=ox.stat"test.lua"
--	ox.utime("xxx",a.mtime)
--	os.execute"ls -l xxx test.lua"
--	ox.unlink"xxx"
	
--	------------------------------------------------------------------------------
--	testing"links"
--	ox.unlink"xxx"
--	io.close(io.open("xxx","w"))
--	myprint(ox.link("xxx","yyy"))
--	myprint(ox.symlink("xxx","zzz"))
--	os.execute"ls -l xxx yyy zzz"
--	myprint("zzz ->",ox.readlink"zzz")
--	myprint("zzz ->",ox.readlink"xxx")
--	ox.unlink"xxx"
--	ox.unlink"yyy"
--	ox.unlink"zzz"
	
	------------------------------------------------------------------------------
	testing"getpasswd"
	function f(x)
	 local a
	 if x==nil then a=ox.getpasswd() else a=ox.getpasswd(x) end
	 if a==nil then
	   myprint(x,"no such user")
	  else
	   myjoin(":",a.name,a.passwd,a.uid,a.gid,a.gecos,a.dir,a.shell)
	 end
	end
	
	f()
	f(ox.getenv"USER")
	f(ox.getenv"LOGNAME")
	f"root"
	f(0)
	f(1234567)
	f"xxx"
	function f(x) myprint(ox.getpasswd(x,"name"),ox.getpasswd(x,"gecos")) end
	f()
	f(nil)
	ox.putenv"USER=root"
	f(ox.getenv"USER")
	
	------------------------------------------------------------------------------
	testing"sysconf"
	a=ox.sysconf() table.foreach(a,myprint)
	testing"pathconf"
	a=ox.pathconf(".") table.foreach(a,myprint)
	
	------------------------------------------------------------------------------
	testing"times"
	a=ox.times()
	for k,v in a do myprint(k,v) end
	myprint"sleeping 2 seconds..."
	ox.sleep(2)
	b=ox.times()
	for k,v in b do myprint(k,v) end
	myprint""
	myprint("elapsed",b.elapsed-a.elapsed)
	myprint("clock",os.clock())
	
	------------------------------------------------------------------------------
	myprint""
	myprint(ox.version)

	return true;
end


return true;


-- luacheia: Below is the original posix.lua wrapper, commented out:
---- posix.lua
---- support code for posix library
---- usage lua -lposix ...
--
--local function so(x)
--	local SOPATH= os.getenv"LUA_SOPATH" or "./"
--	x=assert(loadlib(SOPATH.."l"..x..".so","luaopen_"..x))()
--end
--
--so"posix"
