-----------------------------------------------------------------------------
-- Fast concatenation library
-- LuaSocket toolkit.
-- Author: Roberto Ierusalimschy (typewriting by Diego Nehab ;^)
-- Conformimg to: LTN7, LTN9
-- RCS ID: $Id: concat.lua,v 1.3 2003/07/14 19:52:45 traumwind Exp $
-----------------------------------------------------------------------------

local Private, Public = {}, {}
socket.concat = Public

-----------------------------------------------------------------------------
-- Creates a new concatenation object
-----------------------------------------------------------------------------
function Public.create()
	local concat = {
		addstring = Private.addstring,
		getresult = Private.getresult,
		reset = Private.reset,
		stack = {n = 0}
	}
	return concat
end

-----------------------------------------------------------------------------
-- Adds a string to the concatenation stack
-- Obs: the 2*strlen is to make sure the number of strings stacked is
-- at most log(total bytes)
-----------------------------------------------------------------------------
function Private.addstring(concat, s)
	local stack = concat.stack
    table.insert(stack, s)
    for i = stack.n-1, 1, -1 do
        if string.len(stack[i]) > 2*string.len(stack[i+1]) then break end
        stack[i] = stack[i] .. table.remove(stack)
    end
end

-----------------------------------------------------------------------------
-- Gets the result of the concatenation
-----------------------------------------------------------------------------
function Private.getresult(concat)
	local stack = concat.stack
    for i = stack.n-1, 1, -1 do
        stack[i] = stack[i] .. table.remove(stack)
    end
    return stack[1]
end

-----------------------------------------------------------------------------
-- Resets the buffer
-----------------------------------------------------------------------------
function Private.reset(concat)
	local stack = concat.stack
	for i = 1, stack.n do
		stack[i] = nil
	end
	stack.n = 0
end
