#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

/////////////////////////////////////////////////////////////////////////////
// Name:        wxLuaDebugIO.cpp
// Purpose:     Debugging I/O functions for wxLua
// Author:      J. Winwood
// Created:     May 2002
// Copyright:   (c) 2002 Lomtick Software. All rights reserved.
// Licence:     wxWindows licence
/////////////////////////////////////////////////////////////////////////////

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------

#ifdef __GNUG__
    #pragma implementation "wxluadebugio.h"
#endif

#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

#include "wxLuaDebugIO.h"

// =========================================================

DebugData::DebugData(const char *itemName,
                     const char *itemType,
                     const char *itemValue,
                     int         nReference,
                     int         nIndex,
                     bool        fExpanded)
{
    m_itemName    = itemName;
    m_itemType    = itemType;
    m_itemValue   = itemValue;
    m_nReference  = nReference;
    m_nIndex      = nIndex;
    m_fExpanded   = fExpanded;
}

DebugData::DebugData(const DebugData &treeItem)
{
    m_itemName    = treeItem.m_itemName;
    m_itemType    = treeItem.m_itemType;
    m_itemValue   = treeItem.m_itemValue;
    m_nReference  = treeItem.m_nReference;
    m_nIndex      = treeItem.m_nIndex;
    m_fExpanded   = treeItem.m_fExpanded;
}

// =========================================================

unsigned char  wxLuaDebugIO::ReadByte(wxLuaSocket *pSocket)
{
    unsigned char value;
    pSocket->Read((char *) &value, sizeof(value));
    return value;
}

short wxLuaDebugIO::ReadWord(wxLuaSocket *pSocket)
{
    short value;
    pSocket->Read((char *)&value, sizeof(value));
    return value;
}

unsigned short wxLuaDebugIO::ReadUWord(wxLuaSocket *pSocket)
{
    unsigned short value;
    pSocket->Read((char *)&value, sizeof(value));
    return value;
}

int wxLuaDebugIO::ReadInt(wxLuaSocket *pSocket)
{
    int value;
    pSocket->Read((char *) &value, sizeof(value));
    return value;
}

unsigned int wxLuaDebugIO::ReadUInt(wxLuaSocket *pSocket)
{
    unsigned int value;
    pSocket->Read((char *) &value, sizeof(value));
    return value;
}

long wxLuaDebugIO::ReadLong(wxLuaSocket *pSocket)
{
    long value;
    pSocket->Read((char *) &value, sizeof(value));
    return value;
}

unsigned long  wxLuaDebugIO::ReadULong(wxLuaSocket *pSocket)
{
    unsigned long value;
    pSocket->Read((char *) &value, sizeof(value));
    return value;
}

wxString       wxLuaDebugIO::ReadString(wxLuaSocket *pSocket)
{
    wxString value;
    unsigned int length = ReadInt(pSocket);
    if (length > 0)
    {
        char *buffer = new char[length + 1];
        pSocket->Read(buffer, length);
        buffer[length] = 0;
        value = buffer;
        delete[] buffer;
    }
    return value;
}

wxDebugData *wxLuaDebugIO::ReadDebugData(wxLuaSocket *pSocket)
{
    wxDebugData *pSortedList = new wxDebugData();
    
    try
    {
        size_t idx, idxMax;
        pSocket->Read((char *) &idxMax, sizeof(idxMax));
        
        for (idx = 0; idx < idxMax; ++idx)
        {
            int bufferLength;
            pSocket->Read((char *) &bufferLength, sizeof(bufferLength));
            if (bufferLength > 0)
            {
                char *pBuffer = new char[bufferLength];
				char *pMemory = pBuffer;
                pSocket->Read(pMemory, bufferLength);
                int nReference = *(int *) pMemory;
                pMemory += sizeof(int);
                int nIndex = *(int *) pMemory;
                pMemory += sizeof(int);
                bool fExpanded = (0 != *pMemory++);
                
                const char *pNamePtr = pMemory;
                pMemory += strlen(pNamePtr) + 1;
                const char *pTypePtr = pMemory;
                pMemory += strlen(pTypePtr) + 1;
                const char *pValuePtr = pMemory;
                
                DebugData *pItem = new DebugData(pNamePtr,
                                                 pTypePtr,
                                                 pValuePtr,
                                                 nReference,
                                                 nIndex,
                                                 fExpanded);
                pSortedList->Add(pItem);
                
                delete[] pBuffer;
            }
        }
    }
    catch(wxLuaSocketException & /*e*/)
    {
        delete pSortedList;
        pSortedList = NULL;
    }
    
    return pSortedList;
}

bool wxLuaDebugIO::WriteByte(wxLuaSocket *pSocket, unsigned char value)
{
    bool result = false;
    try
    {        
        pSocket->Write((const char *) &value, sizeof(value));
        result = true;
    }
    catch(wxLuaSocketException & /*e*/)
    {
    }
    return result;
}

bool wxLuaDebugIO::WriteWord(wxLuaSocket *pSocket, short value)
{
    bool result = false;
    try
    {        
        pSocket->Write((const char *) &value, sizeof(value));
        result = true;
    }
    catch(wxLuaSocketException & /*e*/)
    {
    }
    return result;
}

bool wxLuaDebugIO::WriteUWord(wxLuaSocket *pSocket, unsigned short value)
{
    bool result = false;
    try
    {        
        pSocket->Write((const char *) &value, sizeof(value));
        result = true;
    }
    catch(wxLuaSocketException & /*e*/)
    {
    }
    return result;
}

bool wxLuaDebugIO::WriteInt(wxLuaSocket *pSocket, int value)
{
    bool result = false;
    try
    {        
        pSocket->Write((const char *) &value, sizeof(value));
        result = true;
    }
    catch(wxLuaSocketException & /*e*/)
    {
    }
    return result;
}

bool wxLuaDebugIO::WriteUInt(wxLuaSocket *pSocket, unsigned int value)
{
    bool result = false;
    try
    {        
        pSocket->Write((const char *) &value, sizeof(value));
        result = true;
    }
    catch(wxLuaSocketException & /*e*/)
    {
    }
    return result;
}

bool wxLuaDebugIO::WriteLong(wxLuaSocket *pSocket, long value)
{
    bool result = false;
    try
    {        
        pSocket->Write((const char *) &value, sizeof(value));
        result = true;
    }
    catch(wxLuaSocketException & /*e*/)
    {
    }
    return result;
}

bool wxLuaDebugIO::WriteULong(wxLuaSocket *pSocket, unsigned long value)
{
    bool result = false;
    try
    {        
        pSocket->Write((const char *) &value, sizeof(value));
        result = true;
    }
    catch(wxLuaSocketException & /*e*/)
    {
    }
    return result;
}

bool wxLuaDebugIO::WriteString(wxLuaSocket *pSocket, const wxString &value)
{
    bool result = false;
    try
    {        
        int length = value.Length();
        pSocket->Write((const char *) &length, sizeof(length));
        if (length > 0)
        {
            const char *buffer = value.c_str();
            pSocket->Write(buffer, length);
        }
        result = true;
    }
    catch(wxLuaSocketException & /*e*/)
    {
    }
    return result;
}

bool wxLuaDebugIO::WriteDebugData(wxLuaSocket *pSocket, const wxDebugData *pSortedList)
{
    bool result = false;
    size_t idx, idxMax = pSortedList->Count();

    pSocket->Write((const char *) &idxMax, sizeof(idxMax));
        
    for (idx = 0; idx < idxMax; ++idx)
    {
        try
        {
            const DebugData *item = pSortedList->Item(idx);
            
            int nameLength = item->GetName().Length() + 1;
            int typeLength = item->GetType().Length() + 1;
            int valueLength = item->GetValue().Length() + 1;
            
            int bufferLength = (2 * sizeof(int)) + 
                                sizeof(unsigned char) + 
                                nameLength + 
                                typeLength + 
                                valueLength;
            
            unsigned char *pBuffer = new unsigned char[bufferLength];        
            unsigned char *pMemory = pBuffer;

            pSocket->Write((const char *) &bufferLength, sizeof(bufferLength));
            
            *(int *) pMemory = item->GetReference();
            pMemory += sizeof(int);
            
            *(int *) pMemory = item->GetIndex();            
            pMemory += sizeof(int);
            
            *pMemory++ = item->IsExpanded();
            
            memcpy(pMemory, item->GetName(), nameLength);
            pMemory += nameLength;
            
            memcpy(pMemory, item->GetType(), typeLength);
            pMemory += typeLength;
            
            memcpy(pMemory, item->GetValue(), valueLength);                    

            pSocket->Write((const char *) pBuffer, bufferLength);
        
            delete[] pBuffer;
            result = true;
        }
        catch(wxLuaSocketException & /*e*/)
        {
        }
        
        if (result == false)
            break;
    }
    return result;
}

// =========================================================

wxDebugData::wxDebugData() :
    DebugDataArray(SortFunction)    
{
}

wxDebugData::~wxDebugData()
{
	size_t idx;
	for (idx = 0; idx < Count(); ++idx)
	{
		const DebugData *pData = Item(idx);
		delete pData;
	}
}

wxDebugData *wxDebugData::Clone()
{
	wxDebugData *pCloneData = new wxDebugData;
	if (pCloneData != NULL)
	{
		size_t idx;
		for (idx = 0; idx < Count(); ++idx)
		{
			const DebugData *pOldData = Item(idx);
			if (pOldData != NULL)
				pCloneData->Add(new DebugData(*pOldData));
		}
	}
	return pCloneData;
}

int wxDebugData::SortFunction(DebugData *elem1, DebugData *elem2 )
{
    return strcmp(elem1->GetName(), elem2->GetName());
}

// =========================================================

IMPLEMENT_CLASS(wxLuaBuffer, wxObject)

wxLuaBuffer::wxLuaBuffer(const char *fileName, const char *buffer) :
   m_fileName(fileName),
   m_buffer(buffer)
{
}

wxLuaBuffer::~wxLuaBuffer()
{
}

const char *wxLuaBuffer::GetBuffer() const
{
    return m_buffer.c_str();
}

const char *wxLuaBuffer::GetFileName() const
{
    return m_fileName.c_str();
}

int wxLuaBuffer::GetBufferLength() const
{
    return m_buffer.Length();
}
