#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

/////////////////////////////////////////////////////////////////////////////
// Purpose:     Implements wxLuaApp and the wxGetBaseApp function
// Author:      J Winwood
// Created:     14/11/2001
// Modifications: Thanks to Francis Irving for separating out into
//                Library and Standalone modules.
// Copyright:   (c) 2002 Creature Labs. All rights reserved.
// Copyright:   (c) 2001-2002 Lomtick Software. All rights reserved.
// Licence:     wxWindows licence
/////////////////////////////////////////////////////////////////////////////

#ifdef __GNUG__
    #pragma implementation "wxLuaLibrary.cpp"
    #pragma interface "wxLuaLibrary.h"
#endif

#include <wx/wxprec.h>

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

#include "wxLuaLibrary.h"
#include "wxLuaInternals.h"
#include "wxLuaDebugServer.h"
#include "wxLuaStackTree.h"

wxLuaApp::wxLuaApp() :
	m_pStackFrame(NULL)
{
}

wxLuaApp::~wxLuaApp()
{
}

wxLuaApp* wxGetBaseApp()
{ 
	return (wxLuaApp *)wxTheApp; 
}

BEGIN_EVENT_TABLE(wxLuaApp, wxApp)
	EVT_DEBUG_STACK_ENUM(wxLuaApp::OnDebugStackEnum)
	EVT_DEBUG_STACK_ENTRY_ENUM(wxLuaApp::OnDebugStackEntryEnum)
	EVT_DEBUG_TABLE_ENUM(wxLuaApp::OnDebugTableEnum)
END_EVENT_TABLE()

void wxLuaApp::SetStackFrame(StackFrame *pStackFrame)
{
	m_pStackFrame = pStackFrame;
}

void wxLuaApp::OnDebugStackEnum(wxLuaDebugEvent &event)
{
#if wxCHECK_VERSION(2, 3, 0)	
	if (m_pStackFrame != NULL)
		m_pStackFrame->FillStackCombobox(event.GetDebugData());
	else
#endif
		event.Skip();
}

void wxLuaApp::OnDebugStackEntryEnum(wxLuaDebugEvent &event)
{
#if wxCHECK_VERSION(2, 3, 0)	
	if (m_pStackFrame != NULL)
		m_pStackFrame->FillStackEntry(event.GetReference(), event.GetDebugData());
	else
#endif
		event.Skip();
}

void wxLuaApp::OnDebugTableEnum(wxLuaDebugEvent &event)
{
#if wxCHECK_VERSION(2, 3, 0)	
	if (m_pStackFrame != NULL)
		m_pStackFrame->FillTableEntry(event.GetReference(), event.GetDebugData());
	else
#endif
		event.Skip();
}

bool wxLuaApp::OnInit()
{
    return wxApp::OnInit();
}

int  wxLuaApp::OnExit()
{
    return wxApp::OnExit();
}

int LUACALL LuaError(lua_State *L, int status, int top)
{
	int newtop = lua_gettop(L);
	switch(status)
	{
		case 0:		// no error
			break;

		case LUA_ERRRUN:
			{
				wxString strError("wxLua: Runtime error: ");
				if (newtop > top)
					strError += lua_tostring(L, -1);
				strError += wxString("\n");
				wxLuaApp *pApp = wxGetBaseApp();
				pApp->DisplayError(strError);
				lua_settop(L, top);
				status = 0;
			}
			break;

		case LUA_ERRMEM:
			{
				wxString strError("wxLua: Memory allocation error: ");
				if (newtop > top)
					strError += wxString("\n");
				wxLuaApp *pApp = wxGetBaseApp();
				pApp->DisplayError(strError);
				lua_settop(L, top);
				status = 0;
			}
			break;

		case LUA_ERRERR:
			{
				wxString strError("wxLua: Error while running error handler: ");
				if (newtop > top)
					strError += wxString("\n");
				wxLuaApp *pApp = wxGetBaseApp();
				pApp->DisplayError(strError);
				lua_settop(L, top);
				status = 0;
			}
			break;

		case LUA_ERRFILE:
			{
				wxString strError("wxLua: File Error: ");
				if (newtop > top)
					strError += lua_tostring(L, -1);
				strError += wxString("\n");
				wxLuaApp *pApp = wxGetBaseApp();
				pApp->DisplayError(strError);
				lua_settop(L, top);
				status = 0;
			}
			break;

		case LUA_ERRSYNTAX:
			{
				wxString strError("wxLua: Syntax Error: ");
				if (newtop > top)
					strError += lua_tostring(L, -1);
				strError += wxString("\n");
				wxLuaApp *pApp = wxGetBaseApp();
				pApp->DisplayError(strError);
				lua_settop(L, top);
				status = 0;
			}
			break;

		default:
			{
				wxString strError("wxLua: Error: ");
				if (newtop > top)
					strError += lua_tostring(L, -1);
				strError += wxString("\n");
				wxLuaApp *pApp = wxGetBaseApp();
				pApp->DisplayError(strError);
				lua_settop(L, top);
				status = 0;
			}
			break;
	}
	return status;
}

int LUACALL LuaCall(lua_State *L, int narg, int clear) 
{
    int status;
	int top  = lua_gettop(L);
    int base = top - narg;  /* function index */
    lua_pushliteral(L, "_TRACEBACK");
    lua_rawget(L, LUA_GLOBALSINDEX);  /* get traceback function */
    lua_insert(L, base);  /* put it under chunk and args */
    status = lua_pcall(L, narg, (clear ? 0 : LUA_MULTRET), base);
    lua_remove(L, base);  /* remove traceback function */	
	status = LuaError(L, status, top - (narg + 1));
	return status;
}

int LUACALL LuaDoFile(lua_State *L, const char *file)
{
	int top = lua_gettop(L);
	int status = luaL_loadfile(L, file);
	if (status == 0)
	    status = lua_pcall(L, 0, LUA_MULTRET, 0);  /* call main */
	status = LuaError(L, status, top);
	return status;
}

int LUACALL LuaDoBuffer(lua_State *L, const char *buffer, size_t len, const char *name)
{
	int top = lua_gettop(L);
	int status = luaL_loadbuffer(L, buffer, len, name);
	if (status == 0)
	    status = lua_pcall(L, 0, LUA_MULTRET, 0);  /* call main */
	status = LuaError(L, status, top);
	return status;
}

