#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

/////////////////////////////////////////////////////////////////////////////
// Name:        StackTree.cpp
// Purpose:     Display the lua stack in a dialog.
// Author:      J. Winwood
// Created:     February 2002
// Copyright:   (c) 2002 Lomtick Software. All rights reserved.
// Licence:     wxWindows licence
/////////////////////////////////////////////////////////////////////////////

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------

#ifdef __GNUG__
    #pragma implementation "wxluastacktree.h"
#endif

#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

#include "wx/imaglist.h"
#include "wxLuaInternals.h"
#include "wxLuaStackTree.h"

#include "wxLua.h"

#if defined(__WXGTK__) || defined(__WXMOTIF__)
    #include "../wxLua.xpm"
#endif

#if !defined(__WXMSW__) || wxUSE_XPM_IN_MSW
/* Closed folder */
static char * icon1_xpm[] = 
{
    /* width height ncolors chars_per_pixel */
    "16 16 6 1",
    /* colors */
    "   s None  c None",
    ".  c #000000",
    "+  c #c0c0c0",
    "@  c #808080",
    "#  c #ffff00",
    "$  c #ffffff",
    /* pixels */
    "                ",
    "   @@@@@        ",
    "  @#+#+#@       ",
    " @#+#+#+#@@@@@@ ",
    " @$$$$$$$$$$$$@.",
    " @$#+#+#+#+#+#@.",
    " @$+#+#+#+#+#+@.",
    " @$#+#+#+#+#+#@.",
    " @$+#+#+#+#+#+@.",
    " @$#+#+#+#+#+#@.",
    " @$+#+#+#+#+#+@.",
    " @$#+#+#+#+#+#@.",
    " @@@@@@@@@@@@@@.",
    "  ..............",
    "                ",
    "                "
};

/* File */
static char * icon2_xpm[] = 
{
    /* width height ncolors chars_per_pixel */
    "16 16 3 1",
    /* colors */
    "     s None    c None",
    ".    c #000000",
    "+    c #ffffff",
    /* pixels */
    "                ",
    "  ........      ",
    "  .++++++..     ",
    "  .+.+.++.+.    ",
    "  .++++++....   ",
    "  .+.+.+++++.   ",
    "  .+++++++++.   ",
    "  .+.+.+.+.+.   ",
    "  .+++++++++.   ",
    "  .+.+.+.+.+.   ",
    "  .+++++++++.   ",
    "  .+.+.+.+.+.   ",
    "  .+++++++++.   ",
    "  ...........   ",
    "                ",
    "                "
};
#endif

// ----------------------------------------------------------------------------
// StackFrame
// ----------------------------------------------------------------------------

StackFrame::StackFrame(wxWindow		    *parent, 
                       const wxString   &title, 
                       const wxPoint    &pos, 
                       const wxSize     &size, 
					   wxLuaDebugServer *pServer)
       : wxDialog(parent, idSTACK_FRAME, title, pos, size, 
                  wxCAPTION | wxSYSTEM_MENU | wxDIALOG_MODAL | wxRESIZE_BORDER),
         m_luaInterface(NULL),
		 m_pServer(pServer)
{
	Construct();
}

StackFrame::StackFrame(wxWindow		    *parent, 
                       const wxString   &title, 
                       const wxPoint    &pos, 
                       const wxSize     &size, 
                       LuaInterface     *luaInterface)
       : wxDialog(parent, idSTACK_FRAME, title, pos, size, 
                  wxCAPTION | wxSYSTEM_MENU | wxDIALOG_MODAL | wxRESIZE_BORDER),
         m_luaInterface(luaInterface),
		 m_pServer(NULL)
{
	Construct();
}

StackFrame::~StackFrame()
{
	if (m_pServer != NULL)
	    m_pServer->CleanupDebugReferences();		
	wxGetBaseApp()->SetStackFrame(NULL);
}

void StackFrame::Construct()
{
    m_treeSplitter      = NULL;
    m_valueSplitter     = NULL;
	m_scrolledWindow    = NULL;
    m_treeControl       = NULL;
    m_firstValueWindow  = NULL;
    m_secondValueWindow = NULL;

	wxGetBaseApp()->SetStackFrame(this);

    m_stackComboBox = new wxComboBox( this,
                                      idSTACK_COMBOBOX,
                                      "",
                                      wxPoint(0, 0),
                                      wxDefaultSize,
                                      0, 
                                      NULL,
                                      wxCB_DROPDOWN | wxCB_READONLY);
    
    m_scrolledWindow = new wxSplitterScrolledWindow(this, 
                                                    idSCROLLED_WINDOW, 
                                                    wxDefaultPosition,
                                                    wxDefaultSize, 
                                                    wxNO_BORDER | wxCLIP_CHILDREN | wxVSCROLL);
                                                    

#if wxCHECK_VERSION(2, 3, 0)	
    m_dismissButton = new wxButton(this,
                                   idDISMISS_BUTTON,
                                   "Cancel",
                                   wxDefaultPosition,
                                   wxDefaultSize,
                                   wxBU_EXACTFIT);
#else
    m_dismissButton = new wxButton(this,
                                   idDISMISS_BUTTON,
                                   "Cancel",
                                   wxDefaultPosition,
                                   wxDefaultSize);
#endif
    m_dismissButton->SetDefault();
    m_treeSplitter = new wxThinSplitterWindow(m_scrolledWindow, 
                                              idSPLITTER_WINDOW, 
                                              wxDefaultPosition,
                                              wxDefaultSize, 
                                              wxSP_3DBORDER | wxCLIP_CHILDREN);
	m_treeSplitter->SetSashSize(2);

	/* Note the wxTR_ROW_LINES style: draws horizontal lines between items */
#if wxCHECK_VERSION(2, 3, 0)	
    m_treeControl = new StackTree(m_treeSplitter, 
                                  idSTACKTREE_CTRL, 
                                  wxDefaultPosition,
                                  wxDefaultSize, 
                                  wxTR_HAS_BUTTONS | wxTR_NO_LINES | wxNO_BORDER | wxTR_ROW_LINES );
#else
    m_treeControl = new StackTree(m_treeSplitter, 
                                  idSTACKTREE_CTRL, 
                                  wxDefaultPosition,
                                  wxDefaultSize, 
                                  wxTR_HAS_BUTTONS | wxTR_NO_LINES | wxNO_BORDER);
#endif
    m_valueSplitter = new wxThinSplitterWindow(m_treeSplitter, 
                                               idVALUE_SPLITTER_WINDOW, 
                                               wxDefaultPosition,
                                               wxDefaultSize, 
                                               wxSP_NOBORDER | wxCLIP_CHILDREN);
	m_valueSplitter->SetSashSize(2);
    
    m_firstValueWindow = new StackDataWindow(m_valueSplitter, 
                                             idFIRST_VALUE_WINDOW, 
                                             wxDefaultPosition,
                                             wxDefaultSize, 
                                             wxNO_BORDER);
    m_secondValueWindow = new StackDataWindow(m_valueSplitter, 
                                             idSECOND_VALUE_WINDOW, 
                                             wxDefaultPosition,
                                             wxDefaultSize, 
                                             wxNO_BORDER,
											 false);    
                                             
	m_treeSplitter->SetSashPosition(25);
    m_treeSplitter->SetMinimumPaneSize(10);
    m_treeSplitter->SplitVertically(m_treeControl, m_valueSplitter);
    
    m_valueSplitter->SetSashPosition(125);
    m_valueSplitter->SetMinimumPaneSize(10);
    m_valueSplitter->SplitVertically(m_firstValueWindow, m_secondValueWindow);
    
	m_scrolledWindow->SetTargetWindow(m_treeControl);
	m_scrolledWindow->EnableScrolling(FALSE, FALSE);

	// Let the two controls know about each other
	m_firstValueWindow->SetTreeCtrl(m_treeControl);
	m_secondValueWindow->SetTreeCtrl(m_treeControl);
	m_treeControl->SetCompanionWindows(m_firstValueWindow, m_secondValueWindow);
    
    // set the frame icon
#if wxCHECK_VERSION(2, 3, 0)	
    SetIcon(wxICON(LUA));
#endif

    int nWidth, nHeight;
    GetClientSize(&nWidth, &nHeight);
    SizeWindow(nWidth, nHeight);

	if (m_pServer != NULL)
	    m_pServer->EnumerateStack();		
	else
	{	
		wxDebugData *pSortedList = m_luaInterface->BuildStack();
		if (pSortedList != NULL)
		{
			FillStackCombobox(pSortedList);
			delete pSortedList;
		}
	}
}

void StackFrame::FillStackCombobox(const wxDebugData *pSortedList)
{
	size_t idx;

	m_stackComboBox->Clear();

	for (idx = 0; idx < pSortedList->Count(); ++idx)
	{
		const DebugData *item = pSortedList->Item(idx);
		m_stackComboBox->Append(item->GetName(), (void *) item->GetReference());        
	}
	
	if (pSortedList->Count() > 0)
	{
		m_stackComboBox->SetSelection(0);
		SelectStack(0);
	}
}

BEGIN_EVENT_TABLE(StackFrame, wxDialog)
    EVT_SIZE(StackFrame::OnSizeWindow)
    EVT_COMBOBOX(idSTACK_COMBOBOX, StackFrame::OnSelectStack)
    EVT_TREE_ITEM_EXPANDING(idSTACKTREE_CTRL, StackFrame::OnItemExpanding)
    EVT_BUTTON(idDISMISS_BUTTON, StackFrame::OnCloseDialog) 
END_EVENT_TABLE()

void StackFrame::OnSizeWindow(wxSizeEvent &event)
{
    wxSize size = event.GetSize();
    SizeWindow(size.x, size.y);
}

void StackFrame::SizeWindow(int nWidth, int nHeight)
{
    int nComboWidth, nComboHeight, nButtonWidth, nButtonHeight;
    m_stackComboBox->GetSize(&nComboWidth, &nComboHeight);
    m_dismissButton->GetSize(&nButtonWidth, &nButtonHeight);
    nComboHeight += 2; // makes it look a little better
    m_stackComboBox->SetSize(0, -1, nWidth, -1);
    m_scrolledWindow->SetSize(0, 
                              nComboHeight, 
                              nWidth, 
                              nHeight - 5 - (2 * nComboHeight));
    m_dismissButton->SetSize((nWidth / 2) - (nButtonWidth / 2),
                             nHeight - 2 - nComboHeight,
                             nButtonWidth,
                             nButtonHeight);
}

void StackFrame::OnSelectStack(wxCommandEvent &event)
{
    SelectStack(event.GetSelection());
}

void StackFrame::SelectStack(int nCurrentSel)
{
	m_nCurrentSel = nCurrentSel;
	int nEntry = (int) m_stackComboBox->GetClientData(nCurrentSel);
	if (m_pServer != NULL)
	{
		m_pServer->EnumerateStackEntry(nEntry);
	}
	else
	{	    
		wxDebugData *pSortedList = m_luaInterface->BuildStackList(nEntry);
		if (pSortedList != NULL)
		{
			FillStackEntry(nEntry, pSortedList);
			delete pSortedList;
		}
	}
}

void StackFrame::FillStackEntry(int nEntry, const wxDebugData *pSortedList)
{
    m_treeControl->DeleteAllItems();
    wxTreeItemId rootItem = m_treeControl->AddRoot("Locals", -1, -1);
    m_treeControl->AdjustRemoteScrollbars();
    
	size_t idx;
	for (idx = 0; idx < pSortedList->Count(); ++idx)
	{
		const DebugData *item = pSortedList->Item(idx);
	    
		int nOffset = 1;
		if (item->GetReference() != LUA_NOREF)
			nOffset = 0;
	    
		wxTreeItemId treeNode = m_treeControl->AppendItem(rootItem, 
														item->GetName(), 
														nOffset, 
														nOffset, 
														new DebugData(*item));
	    
		if (item->GetReference() != LUA_NOREF)
			m_treeControl->SetItemHasChildren(treeNode);            
	}        

//  If at global scope, process globals
	if (m_nCurrentSel == m_stackComboBox->GetCount() - 1)
	{
		if (m_pServer != NULL)
			m_pServer->EnumerateTable(-1, -1, rootItem);
		else
		{
			wxDebugData *pSortedList = m_luaInterface->GetGlobalData();
			const DebugData *item = pSortedList->Item(0);

			wxTreeItemId treeNode = m_treeControl->AppendItem (rootItem, 
															   item->GetName(), 
															   0, 
															   0,
															   new DebugData(*item));
			
			if (item->GetReference() != LUA_NOREF)
				m_treeControl->SetItemHasChildren(treeNode);

			delete pSortedList;
		}
	}

	m_treeControl->Expand(rootItem);
}

void StackFrame::OnItemExpanding(wxNotifyEvent &event)
{
    ItemExpanding();
}

void StackFrame::ItemExpanding()
{
	wxTreeItemId itemNode = m_treeControl->GetSelection();
	if (itemNode)
	{
		DebugData *pDebugData = (DebugData *)m_treeControl->GetItemData(itemNode);
		if (pDebugData != NULL)
		{
			int nRef = pDebugData->GetReference();
			if (nRef != LUA_NOREF)
			{
				int nIndex = pDebugData->GetIndex() + 1;
				if (!pDebugData->IsExpanded())
				{
					pDebugData->SetExpanded(true);
					if (m_pServer != NULL)
					{
                        m_pServer->EnumerateTable(nRef, nIndex, itemNode);
					}
					else
					{
						// insert items
						wxDebugData *pSortedList = m_luaInterface->BuildTableList(nRef, nIndex);
						if (pSortedList != NULL)
						{
							FillTableEntry(itemNode, pSortedList);
							delete pSortedList;
						}
					}
				}
			}
		}
	}
}

void StackFrame::FillTableEntry(wxTreeItemId itemNode, const wxDebugData *pSortedList)
{
	if (pSortedList->Count() == 0)
		m_treeControl->SetItemHasChildren(itemNode, false);
	else
	{
		size_t count;
		for (count = 0; count < pSortedList->Count(); ++count)
		{
			const DebugData *item = pSortedList->Item(count);

			int nOffset = 1;
			if (item->GetReference() != LUA_NOREF)
				nOffset = 0;
		
			wxTreeItemId treeNode = m_treeControl->AppendItem(itemNode, 
															  item->GetName(), 
															  nOffset, 
															  nOffset, 
															  new DebugData(*item));    
			if (item->GetReference() != LUA_NOREF)
				m_treeControl->SetItemHasChildren(treeNode);
		}
	}
	m_treeControl->Expand(itemNode);
}

void StackFrame::OnCloseDialog(wxCommandEvent &event)
{
    EndModal(0);
}

// ----------------------------------------------------------------------------
// StackTree
// ----------------------------------------------------------------------------

StackTree::StackTree(wxWindow	   *parent, 
                     wxWindowID     id, 
                     const wxPoint &pt,
                     const wxSize  &sz, 
                     long           style):
        wxRemotelyScrolledTreeCtrl(parent, id, pt, sz, style)
{
    m_imageList = new wxImageList(16, 16, TRUE);
#if !defined(__WXMSW__) // || wxUSE_XPM_IN_MSW
    m_imageList->Add(wxIcon(icon1_xpm));
    m_imageList->Add(wxIcon(icon2_xpm));
#elif defined(__WXMSW__)
    m_imageList->Add(wxIcon(wxT("wxICON_SMALL_CLOSED_FOLDER"), wxBITMAP_TYPE_ICO_RESOURCE));
    m_imageList->Add(wxIcon(wxT("wxICON_SMALL_FILE"),		   wxBITMAP_TYPE_ICO_RESOURCE));
#else
#error "Sorry, we don't have icons available for this platforms."
#endif
    SetImageList(m_imageList);
}

IMPLEMENT_CLASS(StackTree, wxRemotelyScrolledTreeCtrl)

BEGIN_EVENT_TABLE(StackTree, wxRemotelyScrolledTreeCtrl)
END_EVENT_TABLE()

StackTree::~StackTree()
{
	SetImageList(NULL);
	delete m_imageList;
}

// ----------------------------------------------------------------------------
// StackDataWindow
// ----------------------------------------------------------------------------

StackDataWindow::StackDataWindow(wxWindow      *parent, 
                                 wxWindowID     id,
                                 const wxPoint &pos,
                                 const wxSize  &sz,
                                 long           style,
                                 bool           fFirst):
      wxTreeCompanionWindow(parent, id, pos, sz, style),
	  m_fFirst(fFirst)
{
	SetBackgroundColour(*wxWHITE);
}

BEGIN_EVENT_TABLE(StackDataWindow, wxTreeCompanionWindow)
END_EVENT_TABLE()

// tulrich: f-ing windows header files!!! #(*&#$
#ifdef DrawText
#undef DrawText
#endif

void StackDataWindow::DrawItem(wxDC &dc, wxTreeItemId id, const wxRect &rect)
{
    if (m_treeControl)
    {
		DebugData *itemData = (DebugData *) m_treeControl->GetItemData(id);

        dc.SetTextForeground(*wxBLACK);
        dc.SetBackgroundMode(wxTRANSPARENT);

        int textWidth, textHeight = 0;
		if (itemData != NULL)
		{
			if (m_fFirst)
				dc.GetTextExtent(itemData->GetType(), &textWidth, &textHeight);
			else
				dc.GetTextExtent(itemData->GetValue(), &textWidth, &textHeight);
		}

        int x = 5;
        int y = rect.GetY() + wxMax(0, (rect.GetHeight() - textHeight) / 2);

		if (itemData != NULL)
		{
			if (m_fFirst)
				dc.DrawText(itemData->GetType(), (wxCoord) x, (wxCoord) y);
			else
				dc.DrawText(itemData->GetValue(), (wxCoord) x, (wxCoord) y);
		}
    }
}

// ----------------------------------------------------------------------------
// LuaInterface (local)
// ----------------------------------------------------------------------------

LuaInterface::LuaInterface(lua_State *luaState) : m_luaState(luaState)
{
}

LuaInterface::~LuaInterface()
{
    size_t idx, idxMax = m_references.GetCount();
    for (idx = 0; idx < idxMax; ++idx)
    {
        int iItem = m_references.Item(idx);
        tremove(m_luaState, iItem);
    }
}

int	LuaInterface::GetStack(int nIndex, lua_Debug *luaDebug)
{
	return lua_getstack(m_luaState, nIndex, luaDebug);
}

bool LuaInterface::GetInfo(const char *what, lua_Debug *luaDebug)
{
	return (lua_getinfo(m_luaState, what, luaDebug) != 0);
}

const char *LuaInterface::GetLocal(lua_Debug *luaDebug, int index)
{
	return lua_getlocal(m_luaState, luaDebug, index);
}

void LuaInterface::GetGlobals()
{
	lua_pushvalue(m_luaState, LUA_GLOBALSINDEX);
}

int	LuaInterface::GetTop()
{
	return lua_gettop(m_luaState);
}

int	LuaInterface::Type(int index)
{
	return lua_type(m_luaState, index);
}

double LuaInterface::ToNumber(int index)
{
	return lua_tonumber(m_luaState, index);
}

int LuaInterface::ToBoolean(int index)
{
	return lua_toboolean(m_luaState, index);
}

const char *LuaInterface::ToString(int index, char * /*buffer*/)
{
	return lua_tostring(m_luaState, index);
}

const void *LuaInterface::ToPointer(int index)
{
	return lua_topointer(m_luaState, index);
}

bool LuaInterface::IsCFunction(int index)
{
	return lua_iscfunction(m_luaState, index) != 0;
}

const void *LuaInterface::ToCFunction(int index)
{
	return (const void *) lua_tocfunction(m_luaState, index);
}

const void *LuaInterface::ToUserData(int index)
{
	return lua_touserdata(m_luaState, index);
}

int LuaInterface::GetTag(int index)
{
	return ttag(m_luaState, index);
}

int LuaInterface::GetN(int index)
{
    return luaL_getn(m_luaState, index);
}

int	LuaInterface::Ref()
{
	int nReference = tinsert(m_luaState, -1);
    m_references.Add(nReference);
	return nReference;
}

int	LuaInterface::GetRef(int ref)
{
	return tget(m_luaState, ref);
}

void LuaInterface::Pop(int count)
{
	lua_pop(m_luaState, count);
}

void LuaInterface::PushNil()
{
	lua_pushnil(m_luaState);
}

void LuaInterface::PushValue(int index)
{
	lua_pushvalue(m_luaState, index);
}

int	LuaInterface::Next(int index)
{
	return lua_next(m_luaState, index);
}

wxDebugData *LuaInterface::BuildStack()
{
    wxDebugData *pSortedList = new wxDebugData();
    
    int       nIndex    = 0;
    lua_Debug luaDebug;
    
    bool fFirst = true;
	while (GetStack(nIndex, &luaDebug) != 0)
    {
		if (GetInfo("Sln", &luaDebug))
	    {
            char buffer[256];
            buffer[0] = 0;
            // skip stack frames that do not have line number
            int  currentLine = luaDebug.currentline;
            if (!fFirst || currentLine != -1)
            {
                if (currentLine == -1)
                    currentLine = 0;
                
                if (luaDebug.name != NULL)
                    sprintf(buffer, "function %s line %u", luaDebug.name, currentLine);
                else
                    sprintf(buffer, "line %u", currentLine);

                DebugData *item = new DebugData(buffer, "", "", nIndex, 0);
                pSortedList->Add(item);
                
                fFirst = false;
            }
        }
        
		++nIndex;
    }
    return pSortedList;
}

wxDebugData *LuaInterface::BuildStackList(int nEntry)
{
    wxDebugData *pSortedList = new wxDebugData();
    if (pSortedList != NULL)
    {
        lua_Debug luaDebug;    
        if (GetStack(nEntry, &luaDebug) != 0)
        {
            int         idx  = 1;
            const char *name = GetLocal(&luaDebug, idx);
            while (name != NULL) 
            {
                bool        fIsTable = false;
                const char *type     = NULL;
                const char *value    = NULL;
                char buffer[256];
    
                switch (Type(-1))
                {
                    case LUA_TNIL:
                        type  = "Nil";
                        value = "nil";
                        break;
    
                    case LUA_TBOOLEAN:
                        type = "Boolean";
                        value = MakeBoolean(ToBoolean(-1));
                        break;

                    case LUA_TNUMBER:
                        type = "Number";
                        value = MakeNumber(ToNumber(-1), buffer);
                        break;
                    
                    case LUA_TSTRING:
                        type  = "String";
                        value = ToString(-1, buffer);
                        break;
    
                    case LUA_TTABLE:
                        type  = "Table";
                        value = GetTableInfo(-1, buffer);
                        fIsTable = true;
                        break;
    
                    case LUA_TFUNCTION:
                        if (IsCFunction(-1))
                        {
                            type  = "C Function";
                            sprintf(buffer, "%p", ToCFunction(-1));
                        }
                        else
                        {
                            type = "Lua Function";
                            sprintf(buffer, "%p", ToPointer(-1));
                        }
                        value = buffer;
                        break;
    
                    case LUA_TUSERDATA:
                        type  = "User Data";
                        value = GetUserDataInfo(-1, buffer);
                        break;
                    
                    case LUA_TLIGHTUSERDATA:
                        type  = "Light User Data";
                        value = GetUserDataInfo(-1, buffer, false);
                        break;

                    case LUA_TTHREAD:
                        type  = "Thread";
                        sprintf(buffer, "%p", ToPointer(-1));
                        value = buffer;
                        break;
                }
    
                int nRef = LUA_NOREF;
    
                if (fIsTable)
				{
                    nRef = Ref();
				}
				else
					Pop(1);
    
                DebugData *item = new DebugData(name, type, value, nRef, 0);
                pSortedList->Add(item);
                
                name = GetLocal(&luaDebug, ++idx);
            }
        }    
    }
    return pSortedList;
}

wxDebugData *LuaInterface::BuildTableList(int nRef, int nIndex)
{
    wxDebugData *pSortedList = new wxDebugData();
    if (pSortedList != NULL && GetRef(nRef) != 0)
    {
        int nTop = GetTop();
    
        // start iterating
        PushNil();    
        while (Next(nTop) != 0)
        {
            bool        fIsTable = false;
            const char *type     = NULL;
            const char *value    = NULL;
            const char *name     = NULL;
    
            char namebuffer[256];
    
            // get the index
            switch (Type(-2))
            {
                case LUA_TNIL:
                    name = "Nil";
                    break;

                case LUA_TBOOLEAN:
                    name = MakeBoolean(ToBoolean(-2));
                    break;
                
                case LUA_TNUMBER:
                    name = MakeNumber(ToNumber(-2), namebuffer);
                    break;
    
                case LUA_TSTRING:
                    name = ToString(-2, namebuffer);
                    break;
    
                case LUA_TTABLE:
                    name = GetTableInfo(-2, namebuffer);
                    break;
    
                case LUA_TFUNCTION:
                    if (IsCFunction(-2))
                    {
                        sprintf(namebuffer, "%p", ToCFunction(-2));
                        name = namebuffer;
                    }
                    else
                    {
                        sprintf(namebuffer, "%p", ToPointer(-2));
                        name = namebuffer;
                    }
                    break;
    
                case LUA_TUSERDATA:
                    name = GetUserDataInfo(-2, namebuffer);
                    break;
                
                case LUA_TLIGHTUSERDATA:
                    name = GetUserDataInfo(-2, namebuffer, false);
                    break;
            }
    
            char valuebuffer[256];
    
            // get the value
            switch (Type(-1))
            {
                case LUA_TNIL:
                    type  = "Nil";
                    value = "nil";
                    break;
    
                case LUA_TBOOLEAN:
                    value = MakeBoolean(ToBoolean(-1));
                    type = "Boolean";
                    break;

                case LUA_TNUMBER:
                    value = MakeNumber(ToNumber(-1), valuebuffer);
                    type = "Number";
                    break;
    
                case LUA_TSTRING:
                    type  = "String";
                    value = ToString(-1, valuebuffer);
                    break;
    
                case LUA_TTABLE:
					type  = "Table";
					value = GetTableInfo(-1, valuebuffer);
					fIsTable = true;
                    break;
    
                case LUA_TFUNCTION:
                    if (IsCFunction(-1))
                    {
                        type  = "C Function";
                        sprintf(valuebuffer, "%p", ToCFunction(-1));
                    }
                    else
                    {
                        type = "Lua Function";
                        sprintf(valuebuffer, "%p", ToPointer(-1));
                    }
                    value = valuebuffer;
                    break;
    
                case LUA_TUSERDATA:
                    type  = "User Data";
                    value = GetUserDataInfo(-1, valuebuffer);
                    break;
                
                case LUA_TLIGHTUSERDATA:
                    type  = "Light User Data";
                    value = GetUserDataInfo(-1, valuebuffer, false);
                    break;
                
                case LUA_TTHREAD:
                    type = "Thread";
                    sprintf(valuebuffer, "%p", ToPointer(-1));
                    value = valuebuffer;
                    break;
            }
    
            int nRef = LUA_NOREF;
            if (fIsTable)
                nRef = Ref();
			else
				Pop(1);
    
            DebugData *item = new DebugData(name, type, value, nRef, nIndex);
            pSortedList->Add(item);    
        }
    
        // remove reference
        Pop(1);
    }
    return pSortedList;
}

const char *LuaInterface::GetTableInfo(int index, char *buffer)
{
    int         nItems = GetN(index); 
    const void *pItem  = ToPointer(index);
    
    if (nItems == 0)
        sprintf(buffer, 
                "%p", 
                pItem);
    else
        sprintf(buffer, 
                "%p approx %u items)", 
                pItem,
                nItems);        
    return buffer;
}

const char *LuaInterface::GetUserDataInfo(int index, char *buffer, bool full)
{
    if (full)
    {
        const char *pTagName = NULL;
        int         nTag     = GetTag(index);
        
        if (nTag != TLUA_NOTAG)
            pTagName = GetLuaTagName(nTag);
        
        if (pTagName != NULL)
            sprintf(buffer, "%p (%s)", ToUserData(index), pTagName);
        else
            sprintf(buffer, "%p (%u)", ToUserData(index), nTag);
    }
    else
        sprintf(buffer, "%p", ToUserData(index));
    return buffer;
}

const char *LuaInterface::MakeNumber(double dnum, char *buff)
{
    long num = (long) dnum;
    if ((double) num == dnum)
    {
        if (num >= 0)
            sprintf(buff, "%lu (0x%lx)", num, num);
        else
            sprintf(buff, "%ld (0x%lx)", num, num);
    }
    else
        sprintf(buff, "%g", dnum);
    return buff;
}

const char *LuaInterface::MakeBoolean(int num)
{
    if (num)
        return "True";
    return "False";
}

wxDebugData *LuaInterface::GetGlobalData()
{
    wxDebugData *pSortedList = new wxDebugData();
    if (pSortedList != NULL)
    {    
        GetGlobals();
		int nRef = Ref();
        pSortedList->Add(new DebugData("Globals", "", "", nRef, 0));
    }
    return pSortedList;
}

