/////////////////////////////////////////////////////////////////////////////
// Purpose:     Standalone wxLua application
// Author:      Francis Irving
// Created:     16/01/2002
// Modifications: J. Winwood. Bug fixes to original wxLua code.
// RCS-ID:      $Id: wxLuaApp.cpp,v 1.3 2003/07/13 22:14:49 tulrich Exp $
// Copyright:   (c) 2002 Creature Labs. All rights reserved.
// Copyright:   (c) 2001-2002 Lomtick Software. All rights reserved.
// Licence:     wxWindows licence
/////////////////////////////////////////////////////////////////////////////
 
#ifdef __GNUG__
    #pragma implementation "wxLuaApp.cpp"
    #pragma interface "wxLuaApp.cpp"
#endif

#include <wx/wxprec.h>

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

#include <wx/splitter.h>
#include <wx/cmdline.h>
#include <wx/fs_mem.h>
#include "../Library/wxLua.h"
#include "wxLuaApp.h"

#if wxCHECK_VERSION(2, 3, 0)	
#include "../Library/wxLuaSplitTree.h"
#include "../Library/wxLuaStackTree.h"
#endif

extern "C"
{
    #include "lualib.h"
}

wxString g_strProgramName;
wxString g_strNetworkName;


static lua_State*	s_existingLuaState = NULL;


bool wxLuaStandaloneApp::OnInit()
{   
    wxString errorStr;
    int      argCount = 0;
    bool     fOk      = false;

#if defined(__WXMSW__) && wxCHECK_VERSION(2, 3, 3)	
	WSADATA wsaData; 
	WORD wVersionRequested = MAKEWORD(1, 1);
	WSAStartup(wVersionRequested, &wsaData);
#endif

	g_strProgramName = argv[0];
	g_strNetworkName = wxGetHostName();

	m_fMemoryBitmapAdded = false;
#if wxCHECK_VERSION(2, 3, 0)	
    m_pDebugTarget = NULL;
#endif    
#ifdef __WXMSW__
    m_pConsole = NULL;
#endif
        
    // Initialize lua
	if (s_existingLuaState)
	{
		// luacheia
		m_luaState = s_existingLuaState;
	}
	else
	{
		m_luaState = lua_open();
	}

    if (m_luaState != NULL)
    {
		if (s_existingLuaState == NULL)	// luacheia
		{
			luaopen_base(m_luaState);
			luaopen_table(m_luaState);
			luaopen_string(m_luaState);
			luaopen_math(m_luaState);
			luaopen_debug(m_luaState);
			luaopen_io(m_luaState);
			luaopen_loadlib(m_luaState);
		}

		// Initialize wxLua
		RegisterClasses(m_luaState);
            
        if (argc == 1)
        {
            int rc = RunEditor();
            
            fOk = CheckError(rc, errorStr);
            if (!fOk)
                DisplayError(errorStr);
        }
        else
        // If there is only one command line argument
#if wxCHECK_VERSION(2, 3, 0) || defined(__WXMSW__)
        if (argc == 2 && *argv[1] == '-')
        {
#if wxCHECK_VERSION(2, 3, 0)	
            // Check if we are launched as a debuggee
            if (strlen(argv[1]) > 2 && argv[1][1] == 'd')
            {
                // Set up for debugging

                char *serverPtr = argv[1] + 2;
                char *portPtr   = strrchr(serverPtr, ':');

                if (portPtr != NULL)
                {
                    *portPtr = 0;
                    int portNumber = atoi(portPtr + 1);

                    m_pDebugTarget = new LuaDebugTarget(m_luaState,
                                                        serverPtr,
                                                        portNumber);
                    if (m_pDebugTarget != NULL)
                        return m_pDebugTarget->Run();
                }
                return false;
            }
#endif        
#ifdef __WXMSW__
            // Check if we are to display the console
            if (strlen(argv[1]) == 2 && argv[1][1] == 'c')
            {
                wxLuaConsole *pConsole = new wxLuaConsole(this, m_luaState);
                if (pConsole != NULL)
                    pConsole->Show(TRUE);

                int rc = RunEditor();
             
                fOk = CheckError(rc, errorStr);
                if (!fOk)
                    DisplayError(errorStr);
            }
            else
#endif			
            // See if a file has been specified that we are to edit
            if (strlen(argv[1]) > 2 && argv[1][1] == 'e')
            {
                const char *fileName = argv[1] + 2;
            
                lua_newtable(m_luaState);            
                lua_pushstring(m_luaState, fileName);
                lua_rawseti(m_luaState, -2, 1);
                lua_pushstring(m_luaState, g_strProgramName);
                lua_rawseti(m_luaState, -2, 0);
                lua_setglobal(m_luaState, "args");
            
                int rc = RunEditor();
            
                fOk = CheckError(rc, errorStr);    
                if (!fOk)
                    DisplayError(errorStr);
            }
            else
            // Otherwise just run the file
            {
                int rc = RunFile(argv[1]);
    
                fOk = CheckError(rc, errorStr);    
                if (!fOk)
                    DisplayError(errorStr);
            }
        } 
        else
#endif        
        // execute the program using argc, argv;
        for (int idx = 1; idx < argc; ++idx)
        {                
#ifdef __WXMSW__
            if (m_pConsole == NULL && strlen(argv[idx]) == 2 && argv[idx][0] == '-' && argv[idx][1] == 'c')
            {
                wxLuaConsole *pConsole = new wxLuaConsole(this, m_luaState);
                if (pConsole != NULL)
                    pConsole->Show(TRUE);
            }
            else
#endif            
            // See if we are to run the specified file with command-line arguments.
            if (strlen(argv[idx]) > 2 && argv[idx][0] == '-' && argv[idx][1] == 'f')
            {
                const char *fileName = argv[idx] + 2;

                lua_newtable(m_luaState);

                // accumulate the arguments into the 'args' table
                for (++idx; idx < argc; ++idx)
                {
                    lua_pushstring(m_luaState, argv[idx]);
                    lua_rawseti(m_luaState, -2, ++argCount);
                }
            
                lua_pushstring(m_luaState, fileName);
                lua_rawseti(m_luaState, -2, 0);

                lua_setglobal(m_luaState, "args");
            
                int rc = RunFile(fileName);
    
                fOk = CheckError(rc, errorStr);    
                if (!fOk)
                    DisplayError(errorStr);
            
                break;
            }
            else
            // See if a file has been specified that we are to edit
            if (strlen(argv[idx]) > 2 && argv[idx][0] == '-' && argv[idx][1] == 'e')
            {
                const char *fileName = argv[idx] + 2;
            
                lua_newtable(m_luaState);
                lua_pushstring(m_luaState, fileName);
                lua_rawseti(m_luaState, -2, 1);
                lua_pushstring(m_luaState, g_strProgramName);
                lua_rawseti(m_luaState, -2, 0);
                lua_setglobal(m_luaState, "args");
            
                int rc = RunEditor();
            
                fOk = CheckError(rc, errorStr);    
                if (!fOk)
                    DisplayError(errorStr);
            
                break;
            }
            else
            {
                int rc = RunFile(argv[idx]);
    
                fOk = CheckError(rc, errorStr);    
                if (!fOk)
                {
                    DisplayError(errorStr);
                    break;
                }
            }
        }
    }
#ifdef __WXMSW__
    return (m_pConsole != NULL) || fOk;
#else
	return fOk;
#endif	
}

#ifndef __WXMSW__
class wxLuaException
{
public:
    // constructors
    wxLuaException(int nReturnCode) :
	m_nReturnCode(nReturnCode)
    {
    }

    wxLuaException(const wxLuaException &e) :
	m_nReturnCode(e.m_nReturnCode)
    {
    }
    
private:
    wxLuaException& operator=(const wxLuaException& e);

private:
    int m_nReturnCode;
};
#endif

int wxLuaStandaloneApp::OnExit()
{    
#if wxCHECK_VERSION(2, 3, 0)	
    // If acting as a debuggee, we have done so disconnect
    // from the debugger.
    if (m_pDebugTarget != NULL)
    {
        m_pDebugTarget->Stop();
        delete m_pDebugTarget;
        m_pDebugTarget = NULL;
    }
#endif

	if (m_fMemoryBitmapAdded)
		wxMemoryFSHandler::RemoveFile("wxLua");
        
    if (m_luaState != NULL)
    {
		// luacheia: restore print function
		lua_dostring(m_luaState, "if wx._old_print then print = wx._old_print end");

        // Clean up wxLua - this also calls lua_close() on the Lua instance.
        CleanupWxLua(m_luaState, false);	// luacheia: don't close Lua!!!
    
        m_luaState = NULL;
    }

    wxLuaApp::OnExit();
        
#if defined(__WXMSW__) && wxCHECK_VERSION(2, 3, 3)	
	WSACleanup ();
#endif

    return 0;
}

int LUACALL wxLuaStackDialog(lua_State *L)
{
#if wxCHECK_VERSION(2, 3, 0)	
	LuaInterface li(L);
	StackFrame *pStackFrame = new StackFrame(NULL,
											 "Stack frame", 
											 wxDefaultPosition, 
											 wxSize(500, 250), 
											 &li);
	pStackFrame->ShowModal();
    pStackFrame->Destroy();
#endif
    return 0;
}

#ifdef __WXMSW__    
int LUACALL wxLuaStandaloneApp::LuaPrint (lua_State *L) 
{
	int		 i;
	wxString stream;
	int		 n = lua_gettop(L);
    
    lua_pushliteral(L, "tostring");
	lua_gettable(L, LUA_GLOBALSINDEX);
	for (i = 1; i <= n; i++) 
	{
		lua_pushvalue(L, -1);  
		lua_pushvalue(L, i);   
        
		LuaCall(L, 1, false);
        
		const char *s = lua_tostring(L, -1);  
		if (s == NULL)
		  terror(L, "wxLua: `tostring' must return a string to `print'");

		if (i > 1) 
			stream.Append("\t");

		stream.Append(s);
		lua_pop(L, 1);  
	}

	stream.Append("\n");

    wxLuaStandaloneApp &pApp = (wxLuaStandaloneApp &)wxGetApp();
    if (pApp.m_pConsole != NULL)
        pApp.m_pConsole->DisplayText(stream);
    else
        wxMessageBox(stream.c_str(), "wxLua");
	return 0;
}
#endif    

void wxLuaStandaloneApp::RegisterClasses(lua_State *L)
{ 
// luacheia  
//	RegisterWxLua(L);

#ifdef __WXMSW__
    lua_pushcfunction(L, wxLuaStandaloneApp::LuaPrint);
	lua_setglobal(L, "print");
#endif    
    lua_pushcfunction(L, wxLuaStackDialog);
    lua_setglobal(L, "wxLuaStackDialog");    
}

int  wxLuaStandaloneApp::RunEditor()
{
    #include "Editor.h"
        
    int rc = LUA_ERRERR;

	wxFileSystem::AddHandler(new wxMemoryFSHandler);

#if defined(__WXGTK__)
    #include "../Examples/bitmaps/wxLua.xpm"   
	wxMemoryFSHandler::AddFile("wxLua", wxImage(wxBITMAP(wxLua)).ConvertToBitmap(), wxBITMAP_TYPE_BMP);
#else    
    wxMemoryFSHandler::AddFile("wxLua", wxBITMAP(logo), wxBITMAP_TYPE_BMP);
#endif    
	m_fMemoryBitmapAdded = true;

    lua_pushcfunction(m_luaState, CreateStandaloneBitmaps);
    lua_setglobal(m_luaState, "standaloneLoadBitmaps");
    
    rc = LuaDoBuffer(m_luaState, 
                       (const char *)wxLuaEditor, 
                       sizeof(wxLuaEditor), 
                       g_strProgramName.c_str());
        
    return rc;
}

int wxLuaStandaloneApp::RunFile(const char *file)
{
    int rc = LUA_ERRERR;
    rc = LuaDoFile(m_luaState, file);
    return rc;
}

bool wxLuaStandaloneApp::CheckError(int rc, wxString &errorStr)
{
    bool fOk = true;
    
    switch (rc)
    {
        case LUA_ERRRUN:
            errorStr = "wxLua: Error while running chunk";
            fOk = false;
            break;
        case LUA_ERRSYNTAX:
            errorStr = "wxLua: Syntax error during pre-compilation";
            fOk = false;
            break;
        case LUA_ERRMEM:
            errorStr = "wxLua: Memory allocation error";
            fOk = false;
            break;
        case LUA_ERRERR:
            errorStr = "wxLua: Generic error or an error occurred while running the error handler";
            fOk = false;
            break;
        case LUA_ERRFILE:
            errorStr = "wxLua: Error occurred while opening file";
            fOk = false;
            break;
    }
    return fOk;
}

void wxLuaStandaloneApp::DisplayError(const wxString &errorStr) const
{
	if (m_pDebugTarget != NULL)
		m_pDebugTarget->DisplayError(errorStr);
	else
	{
#ifdef __WXMSW__
		if (m_pConsole != NULL)
			m_pConsole->DisplayText(errorStr, true);
		else
			wxMessageBox(errorStr.c_str(), "wxLua");
#else
		fprintf(stderr, errorStr);
#endif         
	}
}

//IMPLEMENT_APP(wxLuaStandaloneApp)


// luacheia additions...

IMPLEMENT_APP_NO_MAIN(wxLuaStandaloneApp);


#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H


// loadmodule support & wx.go() implementation


// Enter wxLua.  Doesn't return!  Pass in the wxLua command-line you
// want to execute.  If you don't pass anything in, it will run
// stand-alone interactive.
extern "C" int	wxLua_go(lua_State* L)
{
	const char*	command_line = NULL;

	if (lua_isnil(L, 1)
		|| lua_isstring(L, 1) == 0)
	{
		command_line = "";
	}
	else
	{
		command_line = lua_tostring(L, 1);
	}

#if defined(__WXMSW__)
	char*	cmd_copy = strdup(command_line);
	wxEntry(GetModuleHandle(NULL), NULL, cmd_copy, SW_SHOWNORMAL);
	free(cmd_copy);
#else
	// parse command_line into argc, argv
	wxEntry(argc, argv);
#endif

	return 0;
}


static const struct luaL_reg wxlib[] = {
	{ "go", wxLua_go },
	{ NULL, NULL }
};


extern "C" LUAMODULE_API const char*	luaLM_version()
{
	return LUA_VERSION;
}

extern "C" LUAMODULE_API int luaLM_import(lua_State* L)
{
	// Initialize wxLua
	RegisterWxLua(L);

	s_existingLuaState = L;

	luaL_openlib(L, "wx", wxlib, 0);

	return 0;
}
