/*=============================================*/
/* This code is based on an article in c't 4/97*/
/* see copyright notice below                  */
/* also see original code in 'fuzzsrch.c'      */
/* 'port' as lua-module by                     */
/* Martin Spernau, martin at traumwind dot de  */
/* edited 2003-02-23 for Lua 5.0-beta by e */
/*=============================================*/
/* in Lua do:                                  */
/* thetext = [[A lnger Text to saerch]]        */
/* searchstr = "longer"                        */
/* print(fuzzy(thetext,searchstr))             */
/* -- gives about 54.54                        */
/* while                                       */
/* print(fuzzy(thetext,"lnger"))              */
/* -- gives 100, a complete match              */
/*=============================================*/
/* this Lua extension was built to be used     */
/* with the 'loadmodule' by Thatcher Ulrich    */
/* http://lua-users.org/wiki/LuaBinaryModules  */
/*=============================================*/

/*    Unscharfe Suche im Information Retrieval               */
/*    (C) 1997 Reinhard Rapp                                 */
/*    ANSI C, getestet mit GNU C unter UNIX und MS-DOS,      */
/*    Borland C 5.0 unter Windows 95,                        */
/*    Metrowerks C (CW10) unter MacOS                        */
/*=============================================*/

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#ifndef LUAMODULE_API
# if defined(WIN32) || defined(_WIN32)
#  define LUAMODULE_API __declspec(dllexport)
# else
#  define LUAMODULE_API 
# endif
#endif

#ifdef __cplusplus
extern "C" {
#endif
    
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>


#include <lua.h>
#include <lualib.h>
#include <lauxlib.h>

#ifdef __cplusplus
}
#endif



/* main function, unmodified from original code */
int NGramMatch(char* TextPara, char* SearchStr,
               int SearchStrLen, int NGramLen, int* MaxMatch)
{
  char    NGram[8];
  int     NGramCount;
  int     i, Count;

  NGram[NGramLen] = '\0';
  NGramCount = SearchStrLen - NGramLen + 1;

/* Suchstring in n-Gramme zerlegen und diese im Text suchen */
  for(i = 0, Count = 0, *MaxMatch = 0; i < NGramCount; i++)
    {
      memcpy(NGram, &SearchStr[i], NGramLen);
      
      /* bei Wortzwischenraum weiterruecken */
      if (NGram[NGramLen - 2] == ' ' && NGram[0] != ' ')
          i += NGramLen - 3;
      else
        {
          *MaxMatch  += NGramLen;
          if(strstr(TextPara, NGram)) Count++;
        }
    }
  return Count * NGramLen;  /* gewichten nach n-Gramm-Laenge */
}

    
#ifdef __cplusplus
extern "C" {
#endif
int lfuzzy_initialize(lua_State *L);
#ifdef __cplusplus
}
#endif
static int
wrap_NGramMatch(lua_State *L)
/* 
int NGramMatch(char* TextPara, char* SearchStr,
               int SearchStrLen, int NGramLen, int* MaxMatch)
*/
{
	// args for Lua function
    char*	TextPara;		// the text to search in
    char*	SearchStr;		// the search string
	
	// internals
	int		SearchStrLen;
	int		NGram1Len, NGram2Len;
	int     MatchCount1, MatchCount2;
	int     MaxMatch1, MaxMatch2;
	float   Similarity;

	// get args from stack
	// get the text to search
	if( /*(lua_isnull(L, 1)) ||*/ (lua_isnil(L, 1)) ) {
		lua_pushstring(L, "please supply a text as argument 1");
		lua_error(L);
	} else if (lua_isstring(L, 1)) {
		TextPara = (char*)lua_tostring(L, 1);
	}
	// get the string to search for
    if( /*(lua_isnull(L, 2)) ||*/ (lua_isnil(L, 2)) ) {
		lua_pushstring(L, "please supply a searchstring as argument 2");
		lua_error(L);
	} else if (lua_isstring(L, 2)) {
		SearchStr = (char*)lua_tostring(L, 2);
	}
  
  /* debug... */	
  // printf("matching '%s' against '%s'\n", SearchStr, TextPara);
  
	// lenght of that string
	SearchStrLen = strlen(SearchStr);
	if (SearchStrLen < 3) 
	{
	  lua_pushstring(L, "searchstring too short (min. 3 chars)");
	  lua_error(L);
	}

	NGram1Len = 3;
	// calculate alternative NGramLen
	NGram2Len = (SearchStrLen < 7) ? 2 : 5;


	// call internal function NGramMatch twice
	MatchCount1 = NGramMatch(TextPara, SearchStr,
		SearchStrLen, NGram1Len, &MaxMatch1);
	MatchCount2 = NGramMatch(TextPara, SearchStr,
			SearchStrLen, NGram2Len, &MaxMatch2);
	

	/* calc hit rate */
	Similarity = 100.0
		* (float)(MatchCount1 + MatchCount2)
        / (float)(MaxMatch1 + MaxMatch2);

	
	// and now report the result
  // printf("MatchCount1: %f\nMatchCount2: %f\n", MatchCount1, MatchCount2);
  // printf("and the result is: %f\n", Similarity);	
	lua_pushnumber(L, (float)Similarity);
	return 1;
}

static const struct luaL_reg fuzzylib[] = {
  {"match",    wrap_NGramMatch},
  {NULL, NULL}
  };


int lfuzzy_initialize(lua_State *L)
{
  const char * lib_namespace = "fuzzy";

  luaL_openlib (L, lib_namespace, fuzzylib, 0);
  return 0;
} /* end of lfuzzy_initialize */

/*=========================================*/
/* now comes the special 'loadmodule' code */
/*=========================================*/

LUAMODULE_API
int	luaLM_import(lua_State* L)
{
	return lfuzzy_initialize(L);
}

LUAMODULE_API
const char* luaLM_version(void)
{
	return LUA_VERSION;
}

